@@ -127,7 +127,7 @@ aspath_delimiter_char (u_char type, u_char which)
 }
 
 /* Convert aspath structure to string expression. */
-static char *
+char *
 aspath_make_str_count (struct aspath *as)
 {
   int space;
@@ -623,9 +623,6 @@ aspath_cmp_left (struct aspath *aspath1, struct aspath *aspath2)
     return 0;
   as2 = seg2->asval[0];
 
-  printf ("as1: %d\n", ntohs (as1));
-  printf ("as2: %d\n", ntohs (as2));
-
   if (as1 == as2)
     return 1;
 
