@@ -40,6 +40,7 @@
 #include "bgpd/bgp_community.h"
 #include "bgpd/bgp_debug.h"
 #include "bgpd/bgp_packet.h"
+#include "bgpd/bgp_ecommunity.h"
 
 /* Attribute strings for logging. */
 struct message attr_str [] = 
@@ -236,10 +237,13 @@ attrhash_key_make (struct attr *attr)
   }
 #endif /* HAVE_IPV6 */
 
+  key += attr->mp_nexthop_global_in.s_addr;
   if (attr->aspath)
     key += aspath_key_make (attr->aspath);
   if (attr->community)
     key += community_hash_make (attr->community);
+  if (attr->ecommunity)
+    key += ecommunity_hash_make (attr->ecommunity);
   if (attr->cluster)
     key += cluster_hash_key_make (attr->cluster);
 
@@ -261,8 +265,10 @@ attrhash_cmp (struct attr *attr1, struct attr *attr2)
 #ifdef HAVE_IPV6
       attr1->mp_nexthop_len == attr2->mp_nexthop_len &&
 #endif /* HAVE_IPV6 */
+      IPV4_ADDR_SAME (&attr1->mp_nexthop_global_in, &attr2->mp_nexthop_global_in) &&
       attr1->aspath == attr2->aspath &&
       attr1->community == attr2->community &&
+      attr1->ecommunity == attr2->ecommunity &&
       attr1->cluster == attr2->cluster)
     return 1;
   else
@@ -299,6 +305,13 @@ bgp_attr_intern (struct attr *attr)
       else
 	attr->community->refcnt++;
     }
+  if (attr->ecommunity)
+    {
+      if (! attr->ecommunity->refcnt)
+	attr->ecommunity = ecommunity_intern (attr->ecommunity);
+      else
+	attr->ecommunity->refcnt++;
+    }
   if (attr->cluster)
     {
       if (! attr->cluster->refcnt)
@@ -311,15 +324,6 @@ bgp_attr_intern (struct attr *attr)
   if (find)
     {
       find->refcnt++;
-#if 0
-      if (find->aspath)
-	find->aspath->refcnt++;
-      if (find->community)
-	find->community->refcnt++;
-      if (find->cluster)
-	find->cluster->refcnt++;
-#endif /* 0 */
-
       return find;
     }
 
@@ -372,6 +376,35 @@ bgp_attr_default_intern (u_char origin)
   return new;
 }
 
+struct attr *
+bgp_attr_aggregate_intern (struct bgp *bgp)
+{
+  struct attr attr;
+  struct attr *new;
+
+  memset (&attr, 0, sizeof (struct attr));
+
+  attr.origin = BGP_ORIGIN_IGP;
+  attr.flag |= ATTR_FLAG_BIT (BGP_ATTR_ORIGIN);
+  attr.aspath = aspath_empty ();
+  attr.flag |= ATTR_FLAG_BIT (BGP_ATTR_AS_PATH);
+  attr.weight = 32768;
+#ifdef HAVE_IPV6
+  attr.mp_nexthop_len = 16;
+#endif
+  attr.flag |= ATTR_FLAG_BIT (BGP_ATTR_ATOMIC_AGGREGATE);
+  attr.flag |= ATTR_FLAG_BIT (BGP_ATTR_AGGREGATOR);
+  if (CHECK_FLAG (bgp->config, BGP_CONFIG_CONFEDERATION))
+    attr.aggregator_as = bgp->confederation_id;
+  else
+    attr.aggregator_as = bgp->as;
+  attr.aggregator_addr = bgp->id;
+
+  new = bgp_attr_intern (&attr);
+  aspath_unintern (new->aspath);
+  return new;
+}
+
 /* Free bgp attribute and aspath. */
 void
 bgp_attr_unintern (struct attr *attr)
@@ -379,12 +412,14 @@ bgp_attr_unintern (struct attr *attr)
   struct attr *ret;
   struct aspath *aspath;
   struct community *community;
+  struct ecommunity *ecommunity;
   struct cluster_list *cluster;
 
   /* Decrement attribute reference. */
   attr->refcnt--;
   aspath = attr->aspath;
   community = attr->community;
+  ecommunity = attr->ecommunity;
   cluster = attr->cluster;
 
   /* If reference becomes zero then free attribute object. */
@@ -401,6 +436,8 @@ bgp_attr_unintern (struct attr *attr)
     aspath_unintern (aspath);
   if (community)
     community_unintern (community);
+  if (ecommunity)
+    ecommunity_unintern (ecommunity);
   if (cluster)
     cluster_unintern (cluster);
 }
@@ -412,6 +449,8 @@ bgp_attr_flush (struct attr *attr)
     aspath_free (attr->aspath);
   if (attr->community && ! attr->community->refcnt)
     community_free (attr->community);
+  if (attr->ecommunity && ! attr->ecommunity->refcnt)
+    ecommunity_free (attr->ecommunity);
   if (attr->cluster && ! attr->cluster->refcnt)
     cluster_free (attr->cluster);
 }
@@ -483,7 +522,7 @@ bgp_attr_aspath (struct peer *peer, bgp_size_t length,
 
   /* In case of IBGP, length will be zero. */
   attr->aspath = aspath_parse (stream_pnt (peer->ibuf), length);
-  if (!attr->aspath)
+  if (! attr->aspath)
     {
       zlog (peer->log, LOG_ERR, "Malformed AS path length is %d", length);
       bgp_notify_send (peer, 
@@ -734,6 +773,19 @@ bgp_mp_reach_parse (struct peer *peer, bgp_size_t length, struct attr *attr,
     case 4:
       stream_get (&attr->mp_nexthop_global_in, s, 4);
       break;
+    case 12:
+      {
+	u_int32_t rd_high;
+	u_int32_t rd_low;
+
+	rd_high = stream_getl (s);
+	rd_low = stream_getl (s);
+	/* stream_forward (s, 8); */
+	stream_get (&attr->mp_nexthop_global_in, s, 4);
+	zlog_info ("VPNv4 nexthop recv %ld:%ld:%s",
+		   rd_high, rd_low, inet_ntoa (attr->mp_nexthop_global_in));
+      }
+      break;
 #ifdef HAVE_IPV6
     case 16:
       stream_get (&attr->mp_nexthop_global, s, 16);
@@ -777,9 +829,12 @@ bgp_mp_reach_parse (struct peer *peer, bgp_size_t length, struct attr *attr,
 
   nlri_len = lim - stream_pnt (s);
  
-  ret = nlri_sanity_check (peer, afi, stream_pnt (s), nlri_len);
-  if (ret < 0)
-    return -1;
+  if (safi != BGP_SAFI_VPNV4)
+    {
+      ret = nlri_sanity_check (peer, afi, stream_pnt (s), nlri_len);
+      if (ret < 0)
+	return -1;
+    }
 
   mp_update->afi = afi;
   mp_update->safi = safi;
@@ -811,9 +866,12 @@ bgp_mp_unreach_parse (struct peer *peer, int length,
 
   withdraw_len = lim - stream_pnt (s);
 
-  ret = nlri_sanity_check (peer, afi, stream_pnt (s), withdraw_len);
-  if (ret < 0)
-    return -1;
+  if (safi != BGP_SAFI_VPNV4)
+    {
+      ret = nlri_sanity_check (peer, afi, stream_pnt (s), withdraw_len);
+      if (ret < 0)
+	return -1;
+    }
 
   mp_withdraw->afi = afi;
   mp_withdraw->safi = safi;
@@ -825,6 +883,23 @@ bgp_mp_unreach_parse (struct peer *peer, int length,
   return 0;
 }
 
+/* Extended Community attribute. */
+int
+bgp_attr_ext_communities (struct peer *peer, bgp_size_t length, 
+			  struct attr *attr, u_char flag)
+{
+  if (length == 0)
+    attr->ecommunity = NULL;
+  else
+    {
+      attr->ecommunity = ecommunity_parse (stream_pnt (peer->ibuf), length);
+      stream_forward (peer->ibuf, length);
+    }
+  attr->flag |= ATTR_FLAG_BIT (BGP_ATTR_EXT_COMMUNITIES);
+
+  return 0;
+}
+
 /* Read attribute of update packet.  This function is called from
    bgp_update () in bgpd.c */
 int
@@ -923,6 +998,9 @@ bgp_attr_parse (struct peer *peer, struct attr *attr, bgp_size_t size,
 	case BGP_ATTR_MP_UNREACH_NLRI:
 	  ret = bgp_mp_unreach_parse (peer, length, mp_withdraw);
 	  break;
+	case BGP_ATTR_EXT_COMMUNITIES:
+	  ret = bgp_attr_ext_communities (peer, length, attr, flag);
+	  break;
 	default:
 	  /* Unknown attribute treatment. */
 	  ret = 0;
@@ -1028,7 +1106,8 @@ extern struct peer *peer_self;
 bgp_size_t
 bgp_packet_attribute (struct peer_conf *conf, struct peer *peer,
 		      struct stream *s, struct attr *attr, struct prefix *p,
-		      afi_t afi, safi_t safi, struct peer *from)
+		      afi_t afi, safi_t safi, struct peer *from,
+		      struct prefix_rd *prd, u_char *tag)
 {
   unsigned long cp;
   struct aspath *aspath;
@@ -1097,7 +1176,15 @@ bgp_packet_attribute (struct peer_conf *conf, struct peer *peer,
   stream_putc (s, ATTR_FLAG_TRANS);
   stream_putc (s, BGP_ATTR_NEXT_HOP);
   stream_putc (s, 4);
-  stream_put_ipv4 (s, attr->nexthop.s_addr);
+  if (safi == SAFI_MPLS_VPN)
+    {
+      if (attr->nexthop.s_addr == 0)
+	stream_put_ipv4 (s, peer->nexthop.v4.s_addr);
+      else
+	stream_put_ipv4 (s, attr->nexthop.s_addr);
+    }
+  else
+    stream_put_ipv4 (s, attr->nexthop.s_addr);
 
   /* MED attribute. */
   if (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_MULTI_EXIT_DISC))
@@ -1137,8 +1224,8 @@ bgp_packet_attribute (struct peer_conf *conf, struct peer *peer,
     }
 
   /* Community attribute. */
-  if (CHECK_FLAG (peer->flags, PEER_FLAG_SEND_COMMUNITY) && 
-      (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_COMMUNITIES)))
+  if (CHECK_FLAG (peer->flags, PEER_FLAG_SEND_COMMUNITY) 
+      && (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_COMMUNITIES)))
     {
       if (attr->community->size * 4 > 255)
 	{
@@ -1277,13 +1364,77 @@ bgp_packet_attribute (struct peer_conf *conf, struct peer *peer,
       if (peer->version == BGP_VERSION_MP_4_DRAFT_00)
 	stream_putw_at (s, draftp, (stream_get_putp (s) - draftp) - 2);
     }
+
+  if (p->family == AF_INET && safi == SAFI_MPLS_VPN)
+    {
+      unsigned long sizep;
+      unsigned long draftp = 0;
+
+      stream_putc (s, ATTR_FLAG_OPTIONAL);
+      stream_putc (s, BGP_ATTR_MP_REACH_NLRI);
+      sizep = stream_get_putp (s);
+      stream_putc (s, 0);	/* Length of this attribute. */
+      stream_putw (s, AFI_IP);	/* AFI */
+      stream_putc (s, BGP_SAFI_VPNV4);	/* SAFI */
+
+      stream_putc (s, 12);
+      stream_putl (s, 0);
+      stream_putl (s, 0);
+      stream_put (s, &attr->mp_nexthop_global_in, 4);
+      zlog_info ("VPNv4 nexthop send %s",
+		 inet_ntoa (attr->mp_nexthop_global_in));
+
+      /* SNPA */
+      stream_putc (s, 0);
+
+      /* In case of old draft BGP-4+. */
+      if (peer->version == BGP_VERSION_MP_4_DRAFT_00)
+	{
+	  draftp = stream_get_putp (s);
+	  stream_putw (s, 0);
+	}
+      
+      /* Tag, RD, Prefix write. */
+      stream_putc (s, p->prefixlen + 88);
+      stream_put (s, tag, 3);
+      stream_put (s, prd->val, 8);
+      stream_put (s, &p->u.prefix, PSIZE (p->prefixlen));
+
+      /* Set MP attribute length. */
+      stream_putc_at (s, sizep, (stream_get_putp (s) - sizep) - 1);
+
+      /* In case of old draft BGP-4+. */
+      if (peer->version == BGP_VERSION_MP_4_DRAFT_00)
+	stream_putw_at (s, draftp, (stream_get_putp (s) - draftp) - 2);
+    }
+
+  /* Extended Communities attribute. */
+  if (CHECK_FLAG (peer->flags, PEER_FLAG_SEND_EXT_COMMUNITY) 
+      && (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_EXT_COMMUNITIES)))
+    {
+      if (attr->ecommunity->size * 8 > 255)
+	{
+	  stream_putc (s, ATTR_FLAG_OPTIONAL|ATTR_FLAG_TRANS|ATTR_FLAG_EXTLEN);
+	  stream_putc (s, BGP_ATTR_EXT_COMMUNITIES);
+	  stream_putw (s, attr->ecommunity->size * 8);
+	}
+      else
+	{
+	  stream_putc (s, ATTR_FLAG_OPTIONAL|ATTR_FLAG_TRANS);
+	  stream_putc (s, BGP_ATTR_EXT_COMMUNITIES);
+	  stream_putc (s, attr->ecommunity->size * 8);
+	}
+      stream_put (s, attr->ecommunity->val, attr->ecommunity->size * 8);
+    }
+
   /* Return total size of attribute. */
   return stream_get_putp (s) - cp;
 }
 
 bgp_size_t
 bgp_packet_withdraw (struct peer *peer, struct stream *s, struct prefix *p,
-		     afi_t afi, safi_t safi)
+		     afi_t afi, safi_t safi, struct prefix_rd *prd,
+		     u_char *tag)
 {
   unsigned long cp;
   unsigned long attrlen_pnt;
@@ -1298,10 +1449,26 @@ bgp_packet_withdraw (struct peer *peer, struct stream *s, struct prefix *p,
   stream_putc (s, 0);		/* Length of this attribute. */
 
   stream_putw (s, family2afi (p->family));
-  stream_putc (s, safi);
-  
-  /* Prefix write. */
-  stream_put_prefix (s, p);
+
+  if (safi == SAFI_MPLS_VPN)
+    {
+      /* SAFI */
+      stream_putc (s, BGP_SAFI_VPNV4);
+
+      /* prefix. */
+      stream_putc (s, p->prefixlen + 88);
+      stream_put (s, tag, 3);
+      stream_put (s, prd->val, 8);
+      stream_put (s, &p->u.prefix, PSIZE (p->prefixlen));
+    }
+  else
+    {
+      /* SAFI */
+      stream_putc (s, safi);
+
+      /* prefix */
+      stream_put_prefix (s, p);
+    }
 
   /* Set MP attribute length. */
   size = stream_get_putp (s) - attrlen_pnt - 1;
@@ -1319,5 +1486,169 @@ bgp_attr_init ()
   aspath_init ();
   attrhash_init ();
   community_init ();
+  ecommunity_init ();
   cluster_init ();
 }
+
+/* Make attribute packet. */
+void
+bgp_dump_routes_attr (struct stream *s, struct attr *attr)
+{
+  unsigned long cp;
+  unsigned long len;
+  struct aspath *aspath;
+
+  /* Remember current pointer. */
+  cp = stream_get_putp (s);
+
+  /* Place holder of length. */
+  stream_putw (s, 0);
+
+  /* Origin attribute. */
+  stream_putc (s, ATTR_FLAG_TRANS);
+  stream_putc (s, BGP_ATTR_ORIGIN);
+  stream_putc (s, 1);
+  stream_putc (s, attr->origin);
+
+  aspath = attr->aspath;
+
+  if (aspath->length > 255)
+    {
+      stream_putc (s, ATTR_FLAG_TRANS|ATTR_FLAG_EXTLEN);
+      stream_putc (s, BGP_ATTR_AS_PATH);
+      stream_putw (s, aspath->length);
+    }
+  else
+    {
+      stream_putc (s, ATTR_FLAG_TRANS);
+      stream_putc(s, BGP_ATTR_AS_PATH);
+      stream_putc (s, aspath->length);
+    }
+  stream_put (s, aspath->data, aspath->length);
+
+  /* Nexthop attribute. */
+  stream_putc (s, ATTR_FLAG_TRANS);
+  stream_putc (s, BGP_ATTR_NEXT_HOP);
+  stream_putc (s, 4);
+  stream_put_ipv4 (s, attr->nexthop.s_addr);
+
+  /* MED attribute. */
+  if (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_MULTI_EXIT_DISC))
+    {
+      stream_putc (s, ATTR_FLAG_OPTIONAL);
+      stream_putc (s, BGP_ATTR_MULTI_EXIT_DISC);
+      stream_putc (s, 4);
+      stream_putl (s, attr->med);
+    }
+
+  /* Local preference. */
+  if (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_LOCAL_PREF))
+    {
+      stream_putc (s, ATTR_FLAG_TRANS);
+      stream_putc (s, BGP_ATTR_LOCAL_PREF);
+      stream_putc (s, 4);
+      stream_putl (s, attr->local_pref);
+    }
+
+  /* Atomic aggregate. */
+  if (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_ATOMIC_AGGREGATE))
+    {
+      stream_putc (s, ATTR_FLAG_TRANS);
+      stream_putc (s, BGP_ATTR_ATOMIC_AGGREGATE);
+      stream_putc (s, 0);
+    }
+
+  /* Aggregator. */
+  if (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_AGGREGATOR))
+    {
+      stream_putc (s, ATTR_FLAG_OPTIONAL|ATTR_FLAG_TRANS);
+      stream_putc (s, BGP_ATTR_AGGREGATOR);
+      stream_putc (s, 6);
+      stream_putw (s, attr->aggregator_as);
+      stream_put_ipv4 (s, attr->aggregator_addr.s_addr);
+    }
+
+  /* Community attribute. */
+  if (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_COMMUNITIES))
+    {
+      if (attr->community->size * 4 > 255)
+	{
+	  stream_putc (s, ATTR_FLAG_OPTIONAL|ATTR_FLAG_TRANS|ATTR_FLAG_EXTLEN);
+	  stream_putc (s, BGP_ATTR_COMMUNITIES);
+	  stream_putw (s, attr->community->size * 4);
+	}
+      else
+	{
+	  stream_putc (s, ATTR_FLAG_OPTIONAL|ATTR_FLAG_TRANS);
+	  stream_putc (s, BGP_ATTR_COMMUNITIES);
+	  stream_putc (s, attr->community->size * 4);
+	}
+      stream_put (s, attr->community->val, attr->community->size * 4);
+    }
+
+#if 0
+  /* Route Reflector. */
+  if (peer_sort (peer) == BGP_PEER_IBGP &&
+      conf->bgp->reflector_cnt)
+    {
+      /* Originator ID. */
+      stream_putc (s, ATTR_FLAG_OPTIONAL);
+      stream_putc (s, BGP_ATTR_ORIGINATOR_ID);
+      stream_putc (s, 4);
+
+      /* If this route is other peer's route. */
+      if (from != peer_self)
+	{
+	  stream_put_in_addr (s, &from->remote_id);
+	}
+      else
+	{
+	  if (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_ORIGINATOR_ID))
+	    stream_put_in_addr (s, &attr->originator_id);
+	  else
+	    stream_put_in_addr (s, &conf->bgp->id);
+	}
+
+      /* Cluster list. */
+      stream_putc (s, ATTR_FLAG_OPTIONAL);
+      stream_putc (s, BGP_ATTR_CLUSTER_LIST);
+      
+      if (attr->cluster)
+	{
+	  stream_putc (s, attr->cluster->length + 4);
+	  stream_put (s, attr->cluster->list, attr->cluster->length);
+	}
+      else
+	stream_putc (s, 4);
+
+      /* If this peer configuration's parent BGP has cluster_id. */
+      if (conf->bgp->config & BGP_CONFIG_CLUSTER_ID)
+	stream_put_in_addr (s, &conf->bgp->cluster);
+      else
+	stream_put_in_addr (s, &conf->bgp->id);
+    }
+
+  /* Extended Communities attribute. */
+  if (CHECK_FLAG (peer->flags, PEER_FLAG_SEND_EXT_COMMUNITY) 
+      && (attr->flag & ATTR_FLAG_BIT (BGP_ATTR_EXT_COMMUNITIES)))
+    {
+      if (attr->ecommunity->size * 8 > 255)
+	{
+	  stream_putc (s, ATTR_FLAG_OPTIONAL|ATTR_FLAG_TRANS|ATTR_FLAG_EXTLEN);
+	  stream_putc (s, BGP_ATTR_EXT_COMMUNITIES);
+	  stream_putw (s, attr->ecommunity->size * 8);
+	}
+      else
+	{
+	  stream_putc (s, ATTR_FLAG_OPTIONAL|ATTR_FLAG_TRANS);
+	  stream_putc (s, BGP_ATTR_EXT_COMMUNITIES);
+	  stream_putc (s, attr->ecommunity->size * 8);
+	}
+      stream_put (s, attr->ecommunity->val, attr->ecommunity->size * 8);
+    }
+#endif /* 0 */
+
+  /* Return total size of attribute. */
+  len = stream_get_putp (s) - cp - 2;
+  stream_putw_at (s, cp, len);
+}
