@@ -29,10 +29,13 @@
 #include "thread.h"
 #include "version.h"
 #include "memory.h"
+#include "prefix.h"
 #include "log.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_network.h"
+#include "bgpd/bgp_attr.h"
+#include "bgpd/bgp_mplsvpn.h"
 
 /* bgpd options, we use GNU getopt library. */
 struct option longopts[] = 
@@ -227,10 +230,11 @@ main (int argc, char **argv)
   /* Initializations. */
   srand (time (NULL));
   signal_init ();
-  cmd_init ();
+  cmd_init (1);
   vty_init ();
   memory_init ();
   bgp_init ();
+  bgp_mplsvpn_init ();
   sort_node ();
 
   /* Parse config file. */
