@@ -42,6 +42,7 @@
 #include "bgpd/bgp_aspath.h"
 #include "bgpd/bgp_community.h"
 #include "bgpd/bgp_network.h"
+#include "bgpd/bgp_mplsvpn.h"
 
 int stream_put_prefix (struct stream *, struct prefix *);
 
@@ -394,7 +395,7 @@ bgp_notify_send (struct peer *peer, u_char code, u_char sub_code)
 void
 bgp_update_send (struct peer_conf *conf, struct peer *peer,
 		 struct prefix *p, struct attr *attr, afi_t afi, safi_t safi,
-		 struct peer *from)
+		 struct peer *from, struct prefix_rd *prd, u_char *tag)
 {
   struct stream *s;
   struct stream *packet;
@@ -425,7 +426,7 @@ bgp_update_send (struct peer_conf *conf, struct peer *peer,
   /* Make place for total attribute length.  */
   pos = stream_get_putp (s);
   stream_putw (s, 0);
-  total_attr_len = bgp_packet_attribute (conf, peer, s, attr, p, afi, safi, from);
+  total_attr_len = bgp_packet_attribute (conf, peer, s, attr, p, afi, safi, from, prd, tag);
 
   /* Set Total Path Attribute Length. */
   stream_putw_at (s, pos, total_attr_len);
@@ -453,7 +454,8 @@ bgp_update_send (struct peer_conf *conf, struct peer *peer,
 
 /* Send BGP update packet. */
 void
-bgp_withdraw_send (struct peer *peer, struct prefix *p, afi_t afi, safi_t safi)
+bgp_withdraw_send (struct peer *peer, struct prefix *p, afi_t afi, safi_t safi,
+		   struct prefix_rd *prd, u_char *tag)
 {
   struct stream *s;
   struct stream *packet;
@@ -495,11 +497,12 @@ bgp_withdraw_send (struct peer *peer, struct prefix *p, afi_t afi, safi_t safi)
   /* Make attribute. */
 #ifdef HAVE_IPV6
   if((p->family == AF_INET6)
-     || (p->family == AF_INET && safi == SAFI_MULTICAST))
+     || (p->family == AF_INET && safi == SAFI_MULTICAST)
+     || (p->family == AF_INET && safi == SAFI_MPLS_VPN))
     {
       pos = stream_get_putp (s);
       stream_putw (s, 0);
-      total_attr_len = bgp_packet_withdraw (peer, s, p, afi, safi);
+      total_attr_len = bgp_packet_withdraw (peer, s, p, afi, safi, prd, tag);
 
       /* Set Total Path Attribute Length. */
       stream_putw_at (s, pos, total_attr_len);
@@ -539,6 +542,8 @@ safi2str (safi_t safi)
     return "SAFI_UNICAST";
   else if (safi == SAFI_MULTICAST)
     return "SAFI_MULTICAST";
+  else if (safi == SAFI_MPLS_VPN || safi == BGP_SAFI_VPNV4)
+    return "SAFI_MPLS_VPN";
   else
     return "Unknown SAFI";
 }
@@ -554,6 +559,10 @@ bgp_route_refresh_send (struct peer *peer, afi_t afi, safi_t safi)
   return;
 #endif /* DISABLE_BGP_ANNOUNCE */
 
+  /* Adjust safi code. */
+  if (safi == SAFI_MPLS_VPN)
+    safi = BGP_SAFI_VPNV4;
+
   zlog (peer->log, LOG_INFO, "%s [Refresh:SEND] %s %s", 
 	peer->host, afi2str (afi), safi2str (safi));
 
@@ -972,6 +981,18 @@ bgp_update_receive (struct peer *peer, bgp_size_t size)
 	  && mp_withdraw.safi == SAFI_MULTICAST)
 	nlri_parse (peer, NULL, &mp_withdraw);
     }
+  if (peer->afc[AFI_IP][SAFI_MPLS_VPN])
+    {
+      if (mp_update.length 
+	  && mp_update.afi == AFI_IP 
+	  && mp_update.safi == BGP_SAFI_VPNV4)
+	nlri_parse_vpnv4 (peer, &attr, &mp_update);
+
+      if (mp_withdraw.length 
+	  && mp_withdraw.afi == AFI_IP 
+	  && mp_withdraw.safi == BGP_SAFI_VPNV4)
+	nlri_parse_vpnv4 (peer, NULL, &mp_withdraw);
+    }
 
   /* Everything is done.  We unintern temporary structures which
      interned in bgp_attr_parse(). */
@@ -1059,7 +1080,7 @@ bgp_route_refresh_receive (struct peer *peer, bgp_size_t size)
   u_char reserved;
 
   /* If peer does not have the capability, send notification. */
-  if (! peer->refresh)
+  if (! peer->refresh_nego)
     {
       plog_err (peer->log, "%s [Error] BGP route refresh is not enabled",
 		peer->host);
@@ -1095,16 +1116,23 @@ bgp_route_refresh_receive (struct peer *peer, bgp_size_t size)
       bgp_notify_send (peer, BGP_NOTIFY_CEASE, 0);
       return;
     }
-  if (safi != SAFI_UNICAST && safi != SAFI_MULTICAST)
+  if (safi != SAFI_UNICAST && safi != SAFI_MULTICAST && safi != BGP_SAFI_VPNV4)
     {
       plog_err (peer->log,
-		"%s [Error] Unknown SAFI %d route refresh", safi);
+		"%s [Error] Unknown SAFI %d route refresh",
+		peer->host, safi);
       bgp_notify_send (peer, BGP_NOTIFY_CEASE, 0);
       return;
     }
+  /* Logging. */
+  zlog_info ("%s [Refresh:RECV] AFI: %d SAFI: %d", peer->host, afi, safi);
+
+  /* Adjust safi code. */
+  if (safi == BGP_SAFI_VPNV4)
+    safi = SAFI_MPLS_VPN;
 
   /* Perform route refreshment to the peer */
-  bgp_announce_table (peer);
+  bgp_refresh_table (peer, afi, safi);
 }
 
 /* BGP read utility function. */
@@ -1190,7 +1218,7 @@ bgp_read (struct thread *thread)
   /* BGP type check. */
   if (type != BGP_MSG_OPEN && type != BGP_MSG_UPDATE 
       && type != BGP_MSG_NOTIFY && type != BGP_MSG_KEEPALIVE 
-      && type != BGP_MSG_ROUTE_REFRESH)
+      && type != BGP_MSG_ROUTE_REFRESH && type != BGP_MSG_ROUTE_REFRESH_01)
     {
       plog_err (peer->log,
 		"%s [Error] Unknown BGP packet type %d received",
@@ -1207,7 +1235,8 @@ bgp_read (struct thread *thread)
       || (type == BGP_MSG_UPDATE && size < BGP_MSG_UPDATE_MIN_SIZE)
       || (type == BGP_MSG_NOTIFY && size < BGP_MSG_NOTIFY_MIN_SIZE)
       || (type == BGP_MSG_KEEPALIVE && size != BGP_MSG_KEEPALIVE_MIN_SIZE)
-      || (type == BGP_MSG_ROUTE_REFRESH && size != BGP_MSG_ROUTE_REFRESH_MIN_SIZE))
+      || (type == BGP_MSG_ROUTE_REFRESH && size != BGP_MSG_ROUTE_REFRESH_MIN_SIZE)
+      || (type == BGP_MSG_ROUTE_REFRESH_01 && size != BGP_MSG_ROUTE_REFRESH_MIN_SIZE))
     {
       plog_err (peer->log,
 		"%s [Error] Bad BGP message length %d for BGP type %s",
@@ -1244,6 +1273,7 @@ bgp_read (struct thread *thread)
       bgp_keepalive_receive (peer, size);
       break;
     case BGP_MSG_ROUTE_REFRESH:
+    case BGP_MSG_ROUTE_REFRESH_01:
       bgp_route_refresh_receive (peer, size);
       break;
     }
