@@ -92,18 +92,37 @@ cmp_node (const void *p, const void *q)
   return strcmp (a->string, b->string);
 }
 
+int
+cmp_desc (const void *p, const void *q)
+{
+  struct desc *a = *(struct desc **)p;
+  struct desc *b = *(struct desc **)q;
+
+  return strcmp (a->cmd, b->cmd);
+}
+
 /* Sort each node's command element according to command string. */
 void
 sort_node ()
 {
-  int i;
+  int i, j;
   struct cmd_node *cnode;
-  
+  vector descvec;
+  struct cmd_element *cmd_element;
+
   for (i = 0; i < vector_max (cmdvec); i++) 
     if ((cnode = vector_slot (cmdvec, i)) != NULL)
       {	
 	vector cmd_vector = cnode->cmd_vector;
 	qsort (cmd_vector->index, cmd_vector->max, sizeof (void *), cmp_node);
+
+	for (j = 0; j < vector_max (cmd_vector); j++)
+	  if ((cmd_element = vector_slot (cmd_vector, j)) != NULL)
+	    {
+	      descvec = vector_slot (cmd_element->strvec,
+				     vector_max (cmd_element->strvec) - 1);
+	      qsort (descvec->index, descvec->max, sizeof (void *), cmp_desc);
+	    }
       }
 }
 
@@ -249,7 +268,7 @@ cmd_make_descvec (char *string, char *descstr)
 	}
       if (*cp == '|')
 	{
-	  if (!multiple)
+	  if (! multiple)
 	    {
 	      fprintf (stderr, "Command parse error!: %s\n", string);
 	      exit (1);
@@ -353,7 +372,8 @@ install_element (enum node_type ntype, struct cmd_element *cmd)
 
   if (cnode == NULL) 
     {
-      fprintf (stderr, "Command node doesn't exist, please check it\n");
+      fprintf (stderr, "Command node %d doesn't exist, please check it\n",
+	       ntype);
       exit (1);
     }
 
@@ -396,38 +416,25 @@ int
 config_write_host (struct vty *vty)
 {
   if (host.name)
-    vty_out (vty, "hostname %s%s", host.name,
-	     VTY_NEWLINE);
+    vty_out (vty, "hostname %s%s", host.name, VTY_NEWLINE);
 
-  /* For security reason, only output password configuration to
-     file. */
-  if (vty->type == VTY_FILE)
+  if (host.encrypt)
     {
-      if (host.encrypt)
-	{
-	  if (host.password_encrypt)
-	    vty_out (vty, "password 8 %s%s", 
-		     host.password_encrypt,
-		     VTY_NEWLINE);
-	  if (host.enable_encrypt)
-	    vty_out (vty, "enable password 8 %s%s", 
-		     host.enable_encrypt,
-		     VTY_NEWLINE);
-	}
-      else
-	{
-	  if (host.password)
-	    vty_out (vty, "password %s%s", host.password,
-		     VTY_NEWLINE);
-	  if (host.enable)
-	    vty_out (vty, "enable password %s%s", host.enable,
-		     VTY_NEWLINE);
-	}
-    }      
+      if (host.password_encrypt)
+        vty_out (vty, "password 8 %s%s", host.password_encrypt, VTY_NEWLINE); 
+      if (host.enable_encrypt)
+        vty_out (vty, "enable password 8 %s%s", host.enable_encrypt, VTY_NEWLINE); 
+    }
+  else
+    {
+      if (host.password)
+        vty_out (vty, "password %s%s", host.password, VTY_NEWLINE);
+      if (host.enable)
+        vty_out (vty, "enable password %s%s", host.enable, VTY_NEWLINE);
+    }
 
   if (host.logfile)
-    vty_out (vty, "log file %s%s", host.logfile,
-	     VTY_NEWLINE);
+    vty_out (vty, "log file %s%s", host.logfile, VTY_NEWLINE);
 
   if (host.log_stdout)
     vty_out (vty, "log stdout%s", VTY_NEWLINE);
@@ -691,8 +698,7 @@ cmd_ipv6_match (char *str)
 	    {
 	      if (*(str + 1) != ':' && *(str + 1) != '\0')
 		return no_match;
-	      
-	      colons--;
+     	      colons--;
 	      state = STATE_COLON;
 	    }
 	  else
@@ -720,15 +726,14 @@ cmd_ipv6_match (char *str)
 	    return no_match;
 	  else
 	    {
-	      colons++;
-	      sp = str + 1;
 	      if (*(str + 1) != '\0')
 		colons++;
+	      sp = str + 1;
 	      state = STATE_ADDR;
 	    }
 
 	  double_colon++;
-	  nums += 2;
+	  nums++;
 	  break;
 	case STATE_ADDR:
 	  if (*(str + 1) == ':' || *(str + 1) == '\0')
@@ -785,7 +790,6 @@ cmd_ipv6_prefix_match (char *str)
 	    {
 	      if (*(str + 1) != ':' && *(str + 1) != '\0')
 		return no_match;
-	      
 	      colons--;
 	      state = STATE_COLON;
 	    }
@@ -816,10 +820,9 @@ cmd_ipv6_prefix_match (char *str)
 	    return no_match;
 	  else
 	    {
-	      colons++;
-	      sp = str + 1;
 	      if (*(str + 1) != '\0' && *(str + 1) != '/')
 		colons++;
+	      sp = str + 1;
 
 	      if (*(str + 1) == '/')
 		state = STATE_SLASH;
@@ -828,7 +831,7 @@ cmd_ipv6_prefix_match (char *str)
 	    }
 
 	  double_colon++;
-	  nums += 2;
+	  nums += 1;
 	  break;
 	case STATE_ADDR:
 	  if (*(str + 1) == ':' || *(str + 1) == '\0' || *(str + 1) == '/')
@@ -866,11 +869,6 @@ cmd_ipv6_prefix_match (char *str)
       str++;
     }
 
-#if 0
-  printf("nums=%d\n", nums);
-  printf("colons=%d\n", colons);
-#endif /* 0 */
-
   if (state < STATE_MASK)
     return partly_match;
 
@@ -881,8 +879,13 @@ cmd_ipv6_prefix_match (char *str)
   if (mask < 0 || mask > 128)
     return no_match;
   
+/* I don't know why mask < 13 makes command match partly.
+   Forgive me to make this comments. I Want to set static default route
+   because of lack of function to originate default in ospf6d; sorry
+       yasu
   if (mask < 13)
     return partly_match;
+*/
 
   return exact_match;
 }
@@ -966,9 +969,12 @@ cmd_filter_by_completion (char *command, vector v, int index)
 		str = desc->cmd;
 
 		if (CMD_VARARG (str))
-		  return vararg_match;
-
-		if (CMD_RANGE (str))
+		  {
+		    if (match_type < vararg_match)
+		      match_type = vararg_match;
+		    matched++;
+		  }
+		else if (CMD_RANGE (str))
 		  {
 		    if (cmd_range_match (str, command))
 		      {
@@ -1078,9 +1084,12 @@ cmd_filter_by_string (char *command, vector v, int index)
 		str = desc->cmd;
 
 		if (CMD_VARARG (str))
-		  return vararg_match;
-
-		if (CMD_RANGE (str))
+		  {
+		    if (match_type < vararg_match)
+		      match_type = vararg_match;
+		    matched++;
+		  }
+		else if (CMD_RANGE (str))
 		  {
 		    if (cmd_range_match (str, command))
 		      {
@@ -1204,15 +1213,6 @@ is_cmd_ambiguous (char *command, vector v, int index, enum match_type type)
  	      case ipv6_match:
 		if (CMD_IPV6 (str))
 		  match++;
-#if 0
-		if ((ret = cmd_ipv6_match (command)) != no_match)
-		  {
-		    if (ret == partly_match)
-		      return 2; /* There is incomplete match. */
-
-		    match++;
-		  }
-#endif /* 0*/
 		break;
 	      case ipv6_prefix_match:
 		if ((ret = cmd_ipv6_prefix_match (command)) != no_match)
@@ -1224,14 +1224,6 @@ is_cmd_ambiguous (char *command, vector v, int index, enum match_type type)
 		  }
 		break;
 	      case ipv4_match:
-#if 0
-		if ((ret = cmd_ipv4_match (command)) != no_match)
-		  {
-		    if (ret == partly_match)
-		      return 2; /* There is incomplete match. */
-		    match++;
-		  }
-#endif /* 0 */
 		if (CMD_IPV4 (str))
 		  match++;
 		break;
@@ -1263,20 +1255,20 @@ is_cmd_ambiguous (char *command, vector v, int index, enum match_type type)
 char *
 cmd_entry_function (char *src, char *dst)
 {
+  /* Skip variable arguments. */
+  if (CMD_OPTION (dst) || CMD_VARIABLE (dst) || CMD_VARARG (dst) ||
+      CMD_IPV4 (dst) || CMD_IPV4_PREFIX (dst) || CMD_RANGE (dst))
+    return NULL;
+
   /* In case of 'command \t', given src is NULL string. */
   if (src == NULL)
-    {
-      if (CMD_OPTION (dst) || CMD_VARIABLE (dst) || CMD_VARARG (dst) ||
-	  CMD_IPV4 (dst) || CMD_IPV4_PREFIX (dst) || CMD_RANGE (dst))
-	return NULL;
-      else
-	return dst;
-    }
+    return dst;
 
+  /* Matched with input string. */
   if (strncmp (src, dst, strlen (src)) == 0)
     return dst;
-  else
-    return NULL;
+
+  return NULL;
 }
 
 /* If src matches dst return dst string, otherwise return NULL */
@@ -1486,17 +1478,48 @@ cmd_describe_command (vector vline, struct vty *vty, int *status)
       }
   vector_free (cmd_vector);
 
-#if 1
   if (vector_slot (matchvec, 0) == NULL)
     {
       vector_free (matchvec);
       *status= CMD_ERR_NO_MATCH;
     }
-#endif /* 0 */
 
   return matchvec;
 }
 
+/* Check LCD of matched command. */
+int
+cmd_lcd (char **matched)
+{
+  int i;
+  int j;
+  int lcd = -1;
+  char *s1, *s2;
+  char c1, c2;
+
+  if (matched[0] == NULL || matched[1] == NULL)
+    return 0;
+
+  for (i = 1; matched[i] != NULL; i++)
+    {
+      s1 = matched[i - 1];
+      s2 = matched[i];
+
+      for (j = 0; (c1 = s1[j]) && (c2 = s2[j]); j++)
+	if (c1 != c2)
+	  break;
+
+      if (lcd < 0)
+	lcd = j;
+      else
+	{
+	  if (lcd > j)
+	    lcd = j;
+	}
+    }
+  return lcd;
+}
+
 /* Command line completion support. */
 char **
 cmd_complete_command (vector vline, struct vty *vty, int *status)
@@ -1511,6 +1534,7 @@ cmd_complete_command (vector vline, struct vty *vty, int *status)
   struct desc *desc;
   vector descvec;
   char *command;
+  int lcd;
 
   /* First, filter by preceeding command string */
   for (i = 0; i < index; i++)
@@ -1532,7 +1556,7 @@ cmd_complete_command (vector vline, struct vty *vty, int *status)
 	  return NULL;
 	}
       /*
-      else if (ret == 2)
+	else if (ret == 2)
 	{
 	  vector_free (cmd_vector);
 	  *status = CMD_ERR_NO_MATCH;
@@ -1566,7 +1590,7 @@ cmd_complete_command (vector vline, struct vty *vty, int *status)
 		if ((string = cmd_entry_function (vector_slot (vline, index),
 						  desc->cmd)))
 		  if (cmd_unique_string (matchvec, string))
-		    vector_set (matchvec, string);
+		    vector_set (matchvec, XSTRDUP (MTYPE_TMP, string));
 	      }
 	  }
       }
@@ -1592,49 +1616,61 @@ cmd_complete_command (vector vline, struct vty *vty, int *status)
   if (vector_slot (matchvec, 1) == NULL)
     {
       match_str = (char **) matchvec->index;
-
       vector_only_wrapper_free (matchvec);
       *status = CMD_COMPLETE_FULL_MATCH;
       return match_str;
     }
+  /* Make it sure last element is NULL. */
+  vector_set (matchvec, NULL);
 
-  /* Check complete match or partly match and make lcd of these
-     matches.  Sorry for complication. */
-  {
-    int i;
-    int j;
-    int index;
+  /* Check LCD of matched strings. */
+  if (vector_slot (vline, index) != NULL)
+    {
+      lcd = cmd_lcd ((char **) matchvec->index);
 
-    index = 0;
-    for (i = 0; i < vector_max (matchvec) - 1; i++)
-      {
-	char c1, c2;
-	char *s1 = vector_slot (matchvec, i);
-	char *s2 = vector_slot (matchvec, i + 1);
-	for (j = 0; (c1 = s1[j]) && (c2 = s2[j]); j++)
-	  if (c1 != c2)
-	    break;
-	if (index > j)
-	  index = j;
-      }
-  }
+      if (lcd)
+	{
+	  int len = strlen (vector_slot (vline, index));
+	  
+	  if (len < lcd)
+	    {
+	      char *lcdstr;
+	      
+	      lcdstr = XMALLOC (MTYPE_TMP, lcd + 1);
+	      memcpy (lcdstr, matchvec->index[0], lcd);
+	      lcdstr[lcd] = '\0';
+
+	      /* match_str = (char **) &lcdstr; */
+
+	      /* Free matchvec. */
+	      for (i = 0; i < vector_max (matchvec); i++)
+		{
+		  if (vector_slot (matchvec, i))
+		    XFREE (MTYPE_TMP, vector_slot (matchvec, i));
+		}
+	      vector_free (matchvec);
+
+      	      /* Make new matchvec. */
+	      matchvec = vector_init (INIT_MATCHVEC_SIZE);
+	      vector_set (matchvec, lcdstr);
+	      match_str = (char **) matchvec->index;
+	      vector_only_wrapper_free (matchvec);
+
+	      *status = CMD_COMPLETE_MATCH;
+	      return match_str;
+	    }
+	}
+    }
 
   match_str = (char **) matchvec->index;
   vector_only_wrapper_free (matchvec);
-  *status = CMD_COMPLETE_MATCH;
+  *status = CMD_COMPLETE_LIST_MATCH;
   return match_str;
 }
 
-/* Command line parser. */
-void
-cmd_parse ()
-{
-  ;
-}
-
 /* Execute command by argument vline vector. */
 int
-cmd_execute_command (vector vline, struct vty *vty)
+cmd_execute_command (vector vline, struct vty *vty, struct cmd_element **cmd)
 {
   int i;
   int index;
@@ -1688,10 +1724,6 @@ cmd_execute_command (vector vline, struct vty *vty)
 	  {
 	    matched_element = cmd_element;
 	    matched_count++;
-#ifdef DEBUG
-	vty_out (vty, "DEBUG: match type %d%s", match, VTY_NEWLINE);
-	vty_out (vty, "DEBUG: %s%s", cmd_element->string, VTY_NEWLINE);
-#endif /* DEBUG */
 	  }
 	else
 	  {
@@ -1745,6 +1777,13 @@ cmd_execute_command (vector vline, struct vty *vty)
 	return CMD_ERR_EXEED_ARGC_MAX;
     }
 
+  /* For vtysh execution. */
+  if (cmd)
+    *cmd = matched_element;
+
+  if (matched_element->daemon)
+    return CMD_SUCCESS_DAEMON;
+
   /* Execute matched command. */
   return (*matched_element->func) (matched_element, vty, argc, argv);
 }
@@ -1882,8 +1921,23 @@ config_from_file (struct vty *vty, FILE *fp)
       /* Try again with setting node to CONFIG_NODE */
       if (ret != CMD_SUCCESS && ret != CMD_WARNING)
 	{
-	  vty->node = CONFIG_NODE;
-	  ret = cmd_execute_command_strict (cmdvec, vline, vty);
+	  if (vty->node == KEYCHAIN_KEY_NODE)
+	    {
+	      vty->node = KEYCHAIN_NODE;
+
+	      ret = cmd_execute_command_strict (cmdvec, vline, vty);
+
+	      if (ret != CMD_SUCCESS && ret != CMD_WARNING)
+		{
+		  vty->node = CONFIG_NODE;
+		  ret = cmd_execute_command_strict (cmdvec, vline, vty);
+		}
+	    }
+	  else
+	    {
+	      vty->node = CONFIG_NODE;
+	      ret = cmd_execute_command_strict (cmdvec, vline, vty);
+	    }
 	}	  
 
       cmd_free_strvec (vline);
@@ -1918,7 +1972,8 @@ DEFUN (enable,
        "Turn on privileged mode command\n")
 {
   /* If enable password is NULL, change to ENABLE_NODE */
-  if (host.enable == NULL && host.enable_encrypt == NULL)
+  if ((host.enable == NULL && host.enable_encrypt == NULL) ||
+      vty->type == VTY_SHELL_SERV)
     vty->node = ENABLE_NODE;
   else
     vty->node = AUTH_ENABLE_NODE;
@@ -1947,7 +2002,10 @@ DEFUN (config_exit,
     {
     case VIEW_NODE:
     case ENABLE_NODE:
-      vty->status = VTY_CLOSE;
+      if (vty_shell (vty))
+	exit (0);
+      else
+	vty->status = VTY_CLOSE;
       break;
     case CONFIG_NODE:
       vty->node = ENABLE_NODE;
@@ -1960,11 +2018,16 @@ DEFUN (config_exit,
     case RIPNG_NODE:
     case OSPF_NODE:
     case OSPF6_NODE:
+    case KEYCHAIN_NODE:
     case MASC_NODE:
     case RMAP_NODE:
     case VTY_NODE:
       vty->node = CONFIG_NODE;
       break;
+    case BGP_VPNV4_NODE:
+      vty->node = BGP_NODE;
+    case KEYCHAIN_KEY_NODE:
+      vty->node = KEYCHAIN_NODE;
     default:
       break;
     }
@@ -1998,9 +2061,12 @@ DEFUN (config_end,
     case RIP_NODE:
     case RIPNG_NODE:
     case BGP_NODE:
+    case BGP_VPNV4_NODE:
     case RMAP_NODE:
     case OSPF_NODE:
     case OSPF6_NODE:
+    case KEYCHAIN_NODE:
+    case KEYCHAIN_KEY_NODE:
     case MASC_NODE:
     case VTY_NODE:
       vty->node = ENABLE_NODE;
@@ -2148,16 +2214,28 @@ DEFUN (config_write_terminal,
   int i;
   struct cmd_node *node;
 
-  vty_out (vty, "%sCurrrent Configuration:%s", VTY_NEWLINE,
-	   VTY_NEWLINE);
-  vty_out (vty, "!%s", VTY_NEWLINE);
+  if (vty->type == VTY_SHELL_SERV)
+    {
+      for (i = 0; i < vector_max (cmdvec); i++)
+	if ((node = vector_slot (cmdvec, i)) && node->func && node->vtysh)
+	  {
+	    if ((*node->func) (vty))
+	      vty_out (vty, "!%s", VTY_NEWLINE);
+	  }
+    }
+  else
+    {
+      vty_out (vty, "%sCurrrent Configuration:%s", VTY_NEWLINE,
+	       VTY_NEWLINE);
+      vty_out (vty, "!%s", VTY_NEWLINE);
 
-  for (i = 0; i < vector_max (cmdvec); i++)
-    if ((node = vector_slot (cmdvec, i)) && node->func)
-      {
-	if ((*node->func) (vty))
-	  vty_out (vty, "!%s", VTY_NEWLINE);
-      }
+      for (i = 0; i < vector_max (cmdvec); i++)
+	if ((node = vector_slot (cmdvec, i)) && node->func)
+	  {
+	    if ((*node->func) (vty))
+	      vty_out (vty, "!%s", VTY_NEWLINE);
+	  }
+    }
   return CMD_SUCCESS;
 }
 
@@ -2168,6 +2246,40 @@ ALIAS (config_write_terminal,
        SHOW_STR
        "running configuration\n")
 
+/* Write startup configuration into the terminal. */
+DEFUN (show_startup_config,
+       show_startup_config_cmd,
+       "show startup-config",
+       SHOW_STR
+       "Contentes of startup configuration\n")
+{
+  char buf[BUFSIZ];
+  FILE *confp;
+
+  confp = fopen (host.config, "r");
+  if (confp == NULL)
+    {
+      vty_out (vty, "Can't open configuration file [%s]%s",
+	       host.config, VTY_NEWLINE);
+      return CMD_WARNING;
+    }
+
+  while (fgets (buf, BUFSIZ, confp))
+    {
+      char *cp = buf;
+
+      while (*cp != '\r' && *cp != '\n' && *cp != '\0')
+	cp++;
+      *cp = '\0';
+
+      vty_out (vty, "%s%s", buf, VTY_NEWLINE);
+    }
+
+  fclose (confp);
+
+  return CMD_SUCCESS;
+}
+
 /* Hostname configuration */
 DEFUN (config_hostname, 
        hostname_cmd,
@@ -2203,10 +2315,11 @@ DEFUN (config_no_hostname,
 
 /* VTY interface password set. */
 DEFUN (config_password, password_cmd,
-       "password [CRYPT] PASSWORD",
+       "password (8|) WORD",
        "Assign the terminal connection password\n"
-       "Crypt: 8 for crypt, password string for cleartext\n"
-       "Password string\n")
+       "Specifies a HIDDEN password will follow\n"
+       "dummy string \n"
+       "The HIDDEN line password string\n")
 {
   /* Argument check. */
   if (argc == 0)
@@ -2256,13 +2369,19 @@ DEFUN (config_password, password_cmd,
   return CMD_SUCCESS;
 }
 
+ALIAS (config_password, password_text_cmd,
+       "password LINE",
+       "Assign the terminal connection password\n"
+       "The UNENCRYPTED (cleartext) line password\n")
+
 /* VTY enable password set. */
 DEFUN (config_enable_password, enable_password_cmd,
-       "enable password [CRYPT] PASSWORD",
+       "enable password (8|) WORD",
        "Modify enable password parameters\n"
        "Assign the privileged level password\n"
-       "Crypt: 8 for crypt, password string for cleartext\n"
-       "Password string\n")
+       "Specifies a HIDDEN password will follow\n"
+       "dummy string \n"
+       "The HIDDEN 'enable' password string\n")
 {
   /* Argument check. */
   if (argc == 0)
@@ -2317,6 +2436,31 @@ DEFUN (config_enable_password, enable_password_cmd,
   return CMD_SUCCESS;
 }
 
+ALIAS (config_enable_password,
+       enable_password_text_cmd,
+       "enable password LINE",
+       "Modify enable password parameters\n"
+       "Assign the privileged level password\n"
+       "The UNENCRYPTED (cleartext) 'enable' password\n")
+
+/* VTY enable password delete. */
+DEFUN (no_config_enable_password, no_enable_password_cmd,
+       "no enable password",
+       NO_STR
+       "Modify enable password parameters\n"
+       "Assign the privileged level password\n")
+{
+  if (host.enable)
+    XFREE (0, host.enable);
+  host.enable = NULL;
+
+  if (host.enable_encrypt)
+    XFREE (0, host.enable_encrypt);
+  host.enable_encrypt = NULL;
+
+  return CMD_SUCCESS;
+}
+	
 DEFUN (service_password_encrypt,
        service_password_encrypt_cmd,
        "service password-encryption",
@@ -2621,11 +2765,15 @@ install_default (enum node_type node)
   install_element (node, &config_end_cmd);
   install_element (node, &config_help_cmd);
   install_element (node, &config_list_cmd);
+
+  install_element (node, &config_write_terminal_cmd);
+  install_element (node, &config_write_file_cmd);
+  install_element (node, &config_write_memory_cmd);
 }
 
 /* Initialize command interface. Install basic nodes and commands. */
 void
-cmd_init ()
+cmd_init (int terminal)
 {
   /* Allocate initial top vector of commands. */
   cmdvec = vector_init (VECTOR_MIN_SIZE);
@@ -2647,22 +2795,26 @@ cmd_init ()
   install_node (&config_node, config_write_host);
 
   /* Each node's basic commands. */
-  install_element (VIEW_NODE, &config_exit_cmd);
-  install_element (VIEW_NODE, &config_quit_cmd);
-  install_element (VIEW_NODE, &config_help_cmd);
-  install_element (VIEW_NODE, &config_list_cmd);
-  install_element (VIEW_NODE, &config_enable_cmd);
   install_element (VIEW_NODE, &show_version_cmd);
-  install_element (VIEW_NODE, &config_terminal_length_cmd);
-  install_element (VIEW_NODE, &no_config_terminal_length_cmd);
+  if (terminal)
+    {
+      install_element (VIEW_NODE, &config_list_cmd);
+      install_element (VIEW_NODE, &config_exit_cmd);
+      install_element (VIEW_NODE, &config_quit_cmd);
+      install_element (VIEW_NODE, &config_help_cmd);
+      install_element (VIEW_NODE, &config_enable_cmd);
+      install_element (VIEW_NODE, &config_terminal_length_cmd);
+      install_element (VIEW_NODE, &no_config_terminal_length_cmd);
+    }
 
-  install_default (ENABLE_NODE);
-  install_element (ENABLE_NODE, &config_disable_cmd);
-  install_element (ENABLE_NODE, &config_terminal_cmd);
-  install_element (ENABLE_NODE, &config_write_terminal_cmd);
-  install_element (ENABLE_NODE, &show_running_config_cmd);
-  install_element (ENABLE_NODE, &config_write_file_cmd);
-  install_element (ENABLE_NODE, &config_write_memory_cmd);
+  if (terminal)
+    {
+      install_default (ENABLE_NODE);
+      install_element (ENABLE_NODE, &config_disable_cmd);
+      install_element (ENABLE_NODE, &config_terminal_cmd);
+      install_element (ENABLE_NODE, &show_running_config_cmd);
+    }
+  install_element (ENABLE_NODE, &show_startup_config_cmd);
   install_element (ENABLE_NODE, &copy_runningconfig_startupconfig_cmd);
   install_element (ENABLE_NODE, &show_version_cmd);
   install_element (ENABLE_NODE, &config_terminal_length_cmd);
@@ -2670,18 +2822,21 @@ cmd_init ()
   install_element (ENABLE_NODE, &service_terminal_length_cmd);
   install_element (ENABLE_NODE, &no_service_terminal_length_cmd);
 
-  install_default (CONFIG_NODE);
+  if (terminal)
+    install_default (CONFIG_NODE);
   install_element (CONFIG_NODE, &hostname_cmd);
   install_element (CONFIG_NODE, &no_hostname_cmd);
   install_element (CONFIG_NODE, &password_cmd);
+  install_element (CONFIG_NODE, &password_text_cmd);
   install_element (CONFIG_NODE, &enable_password_cmd);
+  install_element (CONFIG_NODE, &enable_password_text_cmd);
+  install_element (CONFIG_NODE, &no_enable_password_cmd);
   install_element (CONFIG_NODE, &config_log_stdout_cmd);
   install_element (CONFIG_NODE, &no_config_log_stdout_cmd);
   install_element (CONFIG_NODE, &config_log_file_cmd);
   install_element (CONFIG_NODE, &no_config_log_file_cmd);
   install_element (CONFIG_NODE, &config_log_syslog_cmd);
   install_element (CONFIG_NODE, &no_config_log_syslog_cmd);
-
   install_element (CONFIG_NODE, &config_log_trap_cmd);
   install_element (CONFIG_NODE, &no_config_log_trap_cmd);
   install_element (CONFIG_NODE, &config_log_record_priority_cmd);
