@@ -47,6 +47,8 @@ enum
   MTYPE_AS_PATH,
   MTYPE_COMMUNITY,
   MTYPE_COMMUNITY_VAL,
+  MTYPE_ECOMMUNITY,
+  MTYPE_ECOMMUNITY_VAL,
   MTYPE_CLUSTER,
   MTYPE_CLUSTER_VAL,
   MTYPE_BGP_ROUTE,
@@ -70,7 +72,7 @@ enum
   MTYPE_ROUTE_NODE,
   MTYPE_RIPNG_SLOT,
   MTYPE_ACCESS_LIST,
-  MTYPE_FILTER,
+  MTYPE_ACCESS_FILTER,
   MTYPE_PREFIX_LIST,
   MTYPE_PREFIX_LIST_ENTRY,
   MTYPE_ROUTE_MAP,
@@ -120,7 +122,7 @@ enum
   MTYPE_AS_LIST,
   MTYPE_COMMUNITY_ENTRY,
   MTYPE_COMMUNITY_LIST,
-  MTYPE_ZEBRA,
+  MTYPE_ZCLIENT,
   MTYPE_NEXTHOP,
   MTYPE_RTADV_PREFIX,
   MTYPE_IF_RMAP,
@@ -133,6 +135,16 @@ enum
   MTYPE_BGP_STATIC,
   MTYPE_BGP_AGGREGATE,
   MTYPE_BGP_CONFED_LIST,
+  MTYPE_RIP_OFFSET_LIST,
+  MTYPE_KEYCHAIN,
+  MTYPE_KEY,
+  MTYPE_RIP,
+  MTYPE_RIP_INTERFACE,
+  MTYPE_ROUTE,
+  MTYPE_ROUTE_ENTRY,
+  MTYPE_RIP_DISTANCE,
+  MTYPE_BGP_DISTANCE,
+  MTYPE_OSPF_DISTANCE,
   MTYPE_MAX
 };
 
