@@ -51,8 +51,8 @@ ospf6_interface_free (struct ospf6_interface *o6i)
   return;
 }
 
-struct in6_addr *
-ospf6_interface_linklocal_addr (struct interface *ifp)
+static struct in6_addr *
+ospf6_interface_update_linklocal_address (struct interface *ifp)
 {
   listnode n;
   struct connected *c;
@@ -92,6 +92,7 @@ ospf6_interface_create (struct interface *ifp, struct ospf6 *o6)
   o6i->lladdr = (struct in6_addr *) NULL;
   o6i->area = (struct area *) NULL;
   o6i->state = IFS_DOWN;
+  o6i->is_passive = 0;
   o6i->neighbor_list = list_init ();
   ospf6_lsdb_init_interface (o6i);
   o6i->transdelay = 1;
@@ -102,16 +103,11 @@ ospf6_interface_create (struct interface *ifp, struct ospf6 *o6)
   o6i->cost = 1;
   o6i->ifmtu = 1500;
 
-  o6i->prefix_list = list_init ();
-
-  o6i->lsa_seqnum_link = o6i->lsa_seqnum_network
-                       = o6i->lsa_seqnum_intra_prefix
-                       = INITIAL_SEQUENCE_NUMBER;
-
   /* register interface list */
   if (!o6)
     o6 = ospf6_start ();
   list_add_node (ospf6->ospf6_interface_list, o6i);
+  o6i->ospf6 = o6;
 
   /* link both */
   o6i->interface = ifp;
@@ -133,6 +129,8 @@ ospf6_interface_if_add (struct interface *ifp, struct ospf6 *o6)
 
   o6i->if_id = ifp->ifindex;
 
+  ospf6_interface_address_update (ifp);
+
   /* interface start */
   if (o6i->area)
     thread_add_event (master, interface_up, o6i, 0);
@@ -156,6 +154,25 @@ ospf6_interface_if_del (struct interface *ifp, struct ospf6 *o6)
     thread_add_event (master, interface_down, o6i, 0);
 }
 
+void
+ospf6_interface_state_update (struct interface *ifp)
+{
+  struct ospf6_interface *o6i;
+
+  o6i = (struct ospf6_interface *) ifp->info;
+  if (! o6i)
+    return;
+  if (! o6i->area)
+    return;
+
+  if (if_is_up (ifp))
+    thread_add_event (master, interface_up, o6i, 0);
+  else
+    thread_add_event (master, interface_down, o6i, 0);
+
+  return;
+}
+
 void
 ospf6_interface_address_update (struct interface *ifp)
 {
@@ -167,27 +184,14 @@ ospf6_interface_address_update (struct interface *ifp)
   o6i = (struct ospf6_interface *) ifp->info;
 
   /* reset linklocal pointer */
-  o6i->lladdr = ospf6_interface_linklocal_addr (ifp);
+  o6i->lladdr = ospf6_interface_update_linklocal_address (ifp);
 
   /* if area is null, can't make link-lsa */
-  if (!o6i->area)
+  if (! o6i->area)
     return;
 
   /* create new Link-LSA */
-#if 0
-  ospf6_lsdb_interface_update (o6i);
-#else
-  {
-    struct ospf6_lsa *lsa = NULL;
-    lsa = ospf6_make_link_lsa (o6i);
-    if (!lsa)
-      return;
-
-    ospf6_lsa_flood (lsa);
-    ospf6_lsdb_install (lsa);
-    ospf6_lsa_unlock (lsa);
-  }
-#endif
+  ospf6_lsa_update_link (o6i);
 }
 
 void
@@ -293,7 +297,7 @@ show_if (struct vty *vty, struct interface *iface)
   char *updown[3] = {"down", "up", NULL};
   char *type;
 
-  /* check interface type */
+  /* check physical interface type */
   if (if_is_loopback (iface))
     type = "LOOPBACK";
   else if (if_is_broadcast (iface))
@@ -313,7 +317,7 @@ show_if (struct vty *vty, struct interface *iface)
       return 0;
     }
   else
-    ospf6_interface = (struct ospf6_interface *)iface->info;
+    ospf6_interface = (struct ospf6_interface *) iface->info;
 
   vty_out (vty, "  Internet Address:%s", VTY_NEWLINE);
   for (i = listhead (iface->connected); i; nextnode (i))
@@ -351,9 +355,10 @@ show_if (struct vty *vty, struct interface *iface)
   else
     vty_out (vty, "  Not Attached to Area%s", VTY_NEWLINE);
 
-  vty_out (vty, "  State %s, Transmit Delay %lu sec%s",
+  vty_out (vty, "  State %s, Transmit Delay %lu sec, Priority %d%s",
            ifs_name[ospf6_interface->state],
            ospf6_interface->transdelay,
+           ospf6_interface->priority,
 	   VTY_NEWLINE);
   vty_out (vty, "  Timer intervals configured:%s", VTY_NEWLINE);
   vty_out (vty, "   Hello %lu, Dead %lu, Retransmit %lu%s",
@@ -569,6 +574,11 @@ DEFUN (ipv6_ospf6_priority,
   assert (ospf6_interface);
 
   ospf6_interface->priority = strtol (argv[0], NULL, 10);
+
+  if (ospf6_interface->area)
+    ifs_change (dr_election (ospf6_interface), "Priority reconfigured",
+                ospf6_interface);
+
   return CMD_SUCCESS;
 }
 
