@@ -39,9 +39,6 @@ struct ospf6_interface
   /* linklocal address of this I/F */
   struct in6_addr *lladdr;
 
-  /* list of prefixes: struct in6_addr */
-  list prefix_list;
-
   /* Interface ID; same as ifindex */
   u_int32_t if_id;
 
@@ -68,6 +65,9 @@ struct ospf6_interface
   /* Interface State */
   u_char state;
 
+  /* OSPF6 Interface flag */
+  int is_passive;
+
   /* Decision of DR Election */
   u_int32_t dr;
   u_int32_t bdr;
@@ -84,11 +84,6 @@ struct ospf6_interface
   /* Linklocal LSA Database: includes Link-LSA */
   list lsdb;
 
-  /* Sequence number place holder */
-  int32_t       lsa_seqnum_link;
-  int32_t       lsa_seqnum_network;
-  int32_t       lsa_seqnum_intra_prefix;
-
   /* statistics */
   unsigned int ospf6_stat_dr_election;
   unsigned int ospf6_stat_delayed_lsack;
@@ -108,6 +103,7 @@ struct in6_addr *
 
 void ospf6_interface_if_add (struct interface *, struct ospf6 *);
 void ospf6_interface_if_del (struct interface *, struct ospf6 *);
+void ospf6_interface_state_update (struct interface *);
 void ospf6_interface_address_update (struct interface *);
 
 void ospf6_interface_init ();
