@@ -73,6 +73,8 @@ void ospf6_lsdb_collect_type (list, unsigned short, void *);
 
 struct ospf6_lsa*
 ospf6_lsdb_lookup (unsigned short, unsigned long, unsigned long, void *);
+struct ospf6_lsa*
+ospf6_lsdb_lookup_new (u_int16_t, u_int32_t, u_int32_t, struct ospf6 *);
 void ospf6_lsdb_add (struct ospf6_lsa *);
 void ospf6_lsdb_remove (struct ospf6_lsa *);
 
@@ -93,6 +95,17 @@ void ospf6_lsdb_maxage_remove_as (struct ospf6 *);
 void ospf6_lsdb_check_maxage_lsa (struct ospf6 *);
 
 void ospf6_lsdb_interface_update (struct ospf6_interface *);
+void ospf6_lsdb_init ();
+
+/* new */
+#if TEST
+
+struct ospf6_lsdb
+{
+  struct ospf6_list *list;
+};
+
+#endif /*TEST*/
 
 #endif /* OSPF6_LSDB_H */
 
