@@ -83,22 +83,20 @@ ospf6_redistribute_lsid_get (struct prefix_ipv6 *p)
   u_int32_t lsid = 1;
   struct ospf6_lsa *lsa;
   struct prefix_ipv6 prefix6;
-  struct as_external_lsa *aselsa;
-  struct ospf6_prefix *o6_prefix;
+  struct ospf6_as_external_lsa *aselsa;
 
   prefix6.family = AF_INET6;
 
   while (1)
     {
-      lsa = ospf6_lsdb_lookup (htons (LST_AS_EXTERNAL_LSA), htonl (lsid),
+      lsa = ospf6_lsdb_lookup (htons (OSPF6_LSA_TYPE_AS_EXTERNAL), htonl (lsid),
                                ospf6->router_id, ospf6);
       if (! lsa)
         break;
 
-      aselsa = (struct as_external_lsa *) (lsa->lsa_hdr + 1);
-      o6_prefix = (struct ospf6_prefix *) (&aselsa->ase_prefix_len);
-      prefix6.prefixlen = o6_prefix->o6p_prefix_len;
-      ospf6_prefix_in6_addr (o6_prefix, &prefix6.prefix);
+      aselsa = (struct ospf6_as_external_lsa *) (lsa->lsa_hdr + 1);
+      prefix6.prefixlen = aselsa->ospf6_prefix.prefix_length;
+      ospf6_prefix_in6_addr (&aselsa->ospf6_prefix, &prefix6.prefix);
 
       if (prefix_same ((struct prefix *) &prefix6, (struct prefix *) p))
         break;
@@ -114,16 +112,11 @@ ospf6_redistribute_route_add (int type, int ifindex, struct prefix_ipv6 *p)
 {
   char buf[128];
   struct ospf6_redistribute_info *info;
-  struct ospf6_lsa *lsa = NULL;
   int ret;
   struct route_node *rn;
 
-  /* XXX */
   if (type == ZEBRA_ROUTE_CONNECT)
-    {
-      ospf6_redist_connected_route_add (type, ifindex, p);
-      return;
-    }
+    return;
 
   /* set redistribute info */
   info = XMALLOC (MTYPE_OSPF6_OTHER, sizeof (struct ospf6_redistribute_info));
@@ -156,13 +149,7 @@ ospf6_redistribute_route_add (int type, int ifindex, struct prefix_ipv6 *p)
                  p->prefixlen, info->ls_id);
     }
 
-  lsa = ospf6_lsa_create_as_external (info, p);
-  if (!lsa)
-    return;
-
-  ospf6_lsa_flood (lsa);
-  ospf6_lsdb_install (lsa);
-  ospf6_lsa_unlock (lsa);
+  ospf6_lsa_update_as_external (info->ls_id, ospf6);
 }
 
 void
@@ -192,10 +179,11 @@ ospf6_redistribute_route_remove (int type, int ifindex, struct prefix_ipv6 *p)
                  p->prefixlen);
     }
 
-  lsa = ospf6_lsdb_lookup (htons (LST_AS_EXTERNAL_LSA), htonl (info->ls_id),
+  lsa = ospf6_lsdb_lookup (htons (OSPF6_LSA_TYPE_AS_EXTERNAL),
+                           htonl (info->ls_id),
                            ospf6->router_id, (void *) ospf6);
   if (lsa)
-    ospf6_premature_aging (lsa);
+    ospf6_lsa_premature_aging (lsa);
 
   XFREE (MTYPE_OSPF6_OTHER, info);
   rn->info = NULL;
@@ -275,6 +263,45 @@ DEFUN (no_ospf6_redistribute_static,
   return CMD_SUCCESS;
 }
 
+DEFUN (ospf6_redistribute_kernel,
+       ospf6_redistribute_kernel_cmd,
+       "redistribute kernel",
+       "Redistribute\n"
+       "Static route\n")
+{
+  ospf6->redist_kernel = 1;
+  ospf6_zebra_redistribute (ZEBRA_ROUTE_KERNEL);
+  ospf6_redistribute_routemap_unset (ospf6, ZEBRA_ROUTE_KERNEL);
+  return CMD_SUCCESS;
+}
+
+DEFUN (ospf6_redistribute_kernel_routemap,
+       ospf6_redistribute_kernel_routemap_cmd,
+       "redistribute kernel route-map WORD",
+       "Redistribute\n"
+       "Static routes\n"
+       "Route map reference\n"
+       "Pointer to route-map entries\n")
+{
+  ospf6->redist_kernel = 1;
+  ospf6_zebra_redistribute (ZEBRA_ROUTE_KERNEL);
+  ospf6_redistribute_routemap_set (ospf6, ZEBRA_ROUTE_KERNEL, argv[0]);
+  return CMD_SUCCESS;
+}
+
+DEFUN (no_ospf6_redistribute_kernel,
+       no_ospf6_redistribute_kernel_cmd,
+       "no redistribute kernel",
+       NO_STR
+       "Redistribute\n"
+       "Static route\n")
+{
+  ospf6->redist_kernel = 0;
+  ospf6_zebra_no_redistribute (ZEBRA_ROUTE_KERNEL);
+  ospf6_redistribute_routemap_unset (ospf6, ZEBRA_ROUTE_KERNEL);
+  return CMD_SUCCESS;
+}
+
 DEFUN (ospf6_redistribute_connected,
        ospf6_redistribute_connected_cmd,
        "redistribute connected",
@@ -411,6 +438,15 @@ ospf6_redistribute_config_write (struct vty *vty)
         vty_out (vty, " redistribute static%s", VTY_NEWLINE);
     }
 
+  if (ospf6->redist_kernel)
+    {
+      if (ospf6->rmap[ZEBRA_ROUTE_KERNEL].map)
+        vty_out (vty, " redistribute kernel route-map %s%s",
+                 ospf6->rmap[ZEBRA_ROUTE_KERNEL].name, VTY_NEWLINE);
+      else
+        vty_out (vty, " redistribute kernel%s", VTY_NEWLINE);
+    }
+
   if (ospf6->redist_ripng)
     {
       if (ospf6->rmap[ZEBRA_ROUTE_RIPNG].map)
@@ -443,6 +479,9 @@ ospf6_redistribute_init (struct ospf6 *o6)
   install_element (OSPF6_NODE, &ospf6_redistribute_static_cmd);
   install_element (OSPF6_NODE, &ospf6_redistribute_static_routemap_cmd);
   install_element (OSPF6_NODE, &no_ospf6_redistribute_static_cmd);
+  install_element (OSPF6_NODE, &ospf6_redistribute_kernel_cmd);
+  install_element (OSPF6_NODE, &ospf6_redistribute_kernel_routemap_cmd);
+  install_element (OSPF6_NODE, &no_ospf6_redistribute_kernel_cmd);
   install_element (OSPF6_NODE, &ospf6_redistribute_connected_cmd);
   install_element (OSPF6_NODE, &ospf6_redistribute_connected_routemap_cmd);
   install_element (OSPF6_NODE, &no_ospf6_redistribute_connected_cmd);
@@ -477,5 +516,3 @@ ospf6_redistribute_finish (struct ospf6 *o6)
   route_table_finish (o6->redistribute_map);
 }
 
-
-
