@@ -234,7 +234,7 @@ nexthop_add_from_vertex (struct vertex *dst, struct vertex *parent, list l)
 
       /* get LinkLSA of destination router */
       m = list_init ();
-      ospf6_lsdb_collect_type_advrtr (m, htons (LST_LINK_LSA),
+      ospf6_lsdb_collect_type_advrtr (m, htons (OSPF6_LSA_TYPE_LINK),
                                       dst->vtx_rtrid, (void *)o6if);
       /* set nexthop address */
       if (list_isempty (m))
@@ -284,7 +284,7 @@ nexthop_str (struct ospf6_nexthop *nh, char *buf, size_t bufsize)
     ifname = ifp->name;
 
   if (IN6_IS_ADDR_UNSPECIFIED (&nh->ipaddr))
-    snprintf (ipaddr, sizeof (ipaddr), "Link#%lu", nh->ifindex);
+    snprintf (ipaddr, sizeof (ipaddr), "Link#%d", nh->ifindex);
   else
     inet_ntop (AF_INET6, &nh->ipaddr, ipaddr, sizeof (ipaddr));
   if (nh->advrtr)
@@ -633,7 +633,7 @@ ospf6_route_str (struct route_node *node, char *buf, size_t bufsize)
         rtrid = ospf6_route_get_dst_rtrid ((struct prefix_ipv6 *)&node->p);
         ifid = ospf6_route_get_dst_ifid ((struct prefix_ipv6 *)&node->p),
         inet_ntop (AF_INET, &rtrid, tmp, sizeof (tmp));
-        snprintf (dstr, sizeof (dstr), "%s[%ld]", tmp, 
+        snprintf (dstr, sizeof (dstr), "%s[%lu]", tmp, 
 		  (unsigned long) ntohl (ifid));
         break;
 
@@ -672,35 +672,10 @@ ospf6_route_str (struct route_node *node, char *buf, size_t bufsize)
   return buf;
 }
 
-void
-ospf6_route_vty (struct vty *vty, struct route_node *node)
-{
-  char rnbuf[128], nhbuf[128];
-  struct ospf6_route_node_info *info;
-  listnode n;
-  struct ospf6_nexthop *nh;
-
-  ospf6_route_str (node, rnbuf, sizeof (rnbuf));
-
-  info = (struct ospf6_route_node_info *) node->info;
-
-  /* xxx, save entry of no nexthop */
-  if (list_isempty (info->nhlist))
-    vty_out (vty, "%s%s", rnbuf,
-	     VTY_NEWLINE);
-
-  for (n = listhead (info->nhlist); n; nextnode (n))
-    {
-      nh = (struct ospf6_nexthop *) getdata (n);
-      nexthop_str (nh, nhbuf, sizeof (nhbuf));
-      vty_out (vty, "%s %s%s", rnbuf, nhbuf, VTY_NEWLINE);
-    }
-}
-
 void
 ospf6_route_table_vty (struct vty *vty, struct route_node *rn, int detail)
 {
-  char destination[128], nexthop[128];
+  char destination[128], buf[128];
   listnode n;
   struct ospf6_nexthop *nh;
   struct ospf6_route_node_info *info;
@@ -711,13 +686,24 @@ ospf6_route_table_vty (struct vty *vty, struct route_node *rn, int detail)
   for (n = listhead (info->nhlist); n; nextnode (n))
     {
       nh = (struct ospf6_nexthop *) getdata (n);
-      nexthop_str (nh, nexthop, sizeof (nexthop));
-      vty_out (vty, "%-38s %-25s%s", destination, nexthop, VTY_NEWLINE);
-      if (detail)
-        vty_out (vty, "    %s %s %s %s %lu%s",
-                 dtype_string[info->dest_type], "xxx",
-                 info->area->str, ptype_string[info->path_type],
-                 info->cost, VTY_NEWLINE);
+      nexthop_str (nh, buf, sizeof (buf));
+      vty_out (vty, "%-38s %-25s%s", destination, buf, VTY_NEWLINE);
+
+      if (!detail)
+        continue;
+
+      vty_out (vty, "    %s %s %s %s %lu%s",
+               dtype_string[info->dest_type], "xxx",
+               info->area->str, ptype_string[info->path_type],
+               info->cost, VTY_NEWLINE);
+
+      if (! info->ls_origin)
+        continue;
+      vty_out (vty, "    Origin: %s Advrtr:%s LS-ID:%lu%s",
+               lstype_name[typeindex (info->ls_origin->lsa_hdr->lsh_type)],
+               inet_ntop (AF_INET, &info->ls_origin->lsa_hdr->lsh_advrtr,
+                          buf, sizeof (buf)),
+               (unsigned long) ntohl (info->ls_origin->lsa_hdr->lsh_id));
     }
 }
 
@@ -741,7 +727,7 @@ ospf6_route_intra_vty (struct vty *vty, struct route_node *rn, int detail)
       nh = (struct ospf6_nexthop *) getdata (n);
       nexthop_str (nh, nexthop, sizeof (nexthop));
       if (info->dest_type == DTYPE_INTRA_LINK)
-        vty_out (vty, "%-11s %-15s[%3lu] %-25s%s",
+        vty_out (vty, "%-11s %15s[%-3lu] %-25s%s",
                  dtype_string[info->dest_type],
                  destination, ifid, nexthop, VTY_NEWLINE);
       else
@@ -749,8 +735,8 @@ ospf6_route_intra_vty (struct vty *vty, struct route_node *rn, int detail)
                  destination, nexthop, VTY_NEWLINE);
       if (detail)
         vty_out (vty, "    %s %s %s %lu%s",
-                 "xxx",
-                 info->area->str, ptype_string[info->path_type],
+                 "xxx", info->area->str,
+                 ptype_string[info->path_type],
                  info->cost, VTY_NEWLINE);
     }
 }
@@ -815,7 +801,7 @@ ospf6_route_calc (struct thread *thread)
   unsigned long lsadvrtr, lsid;
   struct ospf6_lsa *lsa;
   struct intra_area_prefix_lsa *iaplsa;
-  struct as_external_lsa *aselsa;
+  struct ospf6_as_external_lsa *aselsa;
   struct ospf6_prefix *o6p;
   int prefix_count, i;
   list pll; /* prefix LSA list */
@@ -850,7 +836,7 @@ ospf6_route_calc (struct thread *thread)
 #endif /*0*/
 
       /* get prefix LSA */
-      lstype = htons (LST_INTRA_AREA_PREFIX_LSA);
+      lstype = htons (OSPF6_LSA_TYPE_INTRA_PREFIX);
       lsid = ospf6_route_get_dst_ifid ((struct prefix_ipv6 *)&rn->p);
       lsadvrtr = ospf6_route_get_dst_rtrid ((struct prefix_ipv6 *)&rn->p);
       lsa = ospf6_lsdb_lookup (lstype, lsid, lsadvrtr, area);
@@ -883,14 +869,14 @@ ospf6_route_calc (struct thread *thread)
           memset (&prefix, 0, sizeof (prefix));
           prefix.family = AF_INET6;
           ospf6_prefix_in6_addr (o6p, &prefix.prefix);
-          prefix.prefixlen = o6p->o6p_prefix_len;
+          prefix.prefixlen = o6p->prefix_length;
 
           /* set newinfo */
           memset (&newinfo, 0, sizeof (newinfo));
           newinfo.dest_type = DTYPE_PREFIX;
           newinfo.area = area;
           newinfo.path_type = PTYPE_INTRA;
-          newinfo.cost = info->cost + ntohs (o6p->o6p_prefix_metric);
+          newinfo.cost = info->cost + ntohs (o6p->prefix_metric);
           newinfo.nhlist = info->nhlist;
 
           /* log */
@@ -936,7 +922,7 @@ ospf6_route_calc (struct thread *thread)
 
       /* get prefix LSA */
       pll = list_init ();
-      lstype = htons (LST_INTRA_AREA_PREFIX_LSA);
+      lstype = htons (OSPF6_LSA_TYPE_INTRA_PREFIX);
       lsadvrtr = ospf6_route_get_dst_rtrid ((struct prefix_ipv6 *)&rn->p);
       ospf6_lsdb_collect_type_advrtr (pll, lstype, lsadvrtr, area);
       if (list_isempty (pll))
@@ -972,14 +958,14 @@ ospf6_route_calc (struct thread *thread)
               memset (&prefix, 0, sizeof (prefix));
               prefix.family = AF_INET6;
               ospf6_prefix_in6_addr (o6p, &prefix.prefix);
-              prefix.prefixlen = o6p->o6p_prefix_len;
+              prefix.prefixlen = o6p->prefix_length;
 
               /* set newinfo */
               memset (&newinfo, 0, sizeof (newinfo));
               newinfo.dest_type = DTYPE_PREFIX;
               newinfo.area = area;
               newinfo.path_type = PTYPE_INTRA;
-              newinfo.cost = info->cost + ntohs (o6p->o6p_prefix_metric);
+              newinfo.cost = info->cost + ntohs (o6p->prefix_metric);
               newinfo.nhlist = info->nhlist;
 
               /* log */
@@ -1021,21 +1007,14 @@ ospf6_route_calc (struct thread *thread)
 
       /* log */
       ospf6_route_str (rn, rn_str, sizeof (rn_str));
-#if 0
-      if (IS_OSPF6_DUMP_ROUTE)
-        zlog_info ("  Check %s's route", rn_str);
-#endif /*0*/
 
       /* get external LSAs */
       pll = list_init ();
-      lstype = htons (LST_AS_EXTERNAL_LSA);
+      lstype = htons (OSPF6_LSA_TYPE_AS_EXTERNAL);
       lsadvrtr = ospf6_route_get_dst_rtrid ((struct prefix_ipv6 *)&rn->p);
       ospf6_lsdb_collect_type_advrtr (pll, lstype, lsadvrtr, ospf6);
       if (list_isempty (pll))
         {
-#if 0
-          zlog_warn ("  *** can't find external LSA for %s", rn_str);
-#endif /*0*/
           list_delete_all (pll);
           continue;
         }
@@ -1044,16 +1023,13 @@ ospf6_route_calc (struct thread *thread)
       for (ln = listhead (pll); ln; nextnode (ln))
         {
           lsa = (struct ospf6_lsa *) getdata (ln);
-          aselsa = (struct as_external_lsa *) (lsa->lsa_hdr + 1);
-
-          /* point OSPF6 prefix */
-          o6p = (struct ospf6_prefix *) (&aselsa->ase_prefix_len);
+          aselsa = (struct ospf6_as_external_lsa *) (lsa->lsa_hdr + 1);
 
           /* set prefix destination */
           memset (&prefix, 0, sizeof (prefix));
           prefix.family = AF_INET6;
-          ospf6_prefix_in6_addr (o6p, &prefix.prefix);
-          prefix.prefixlen = o6p->o6p_prefix_len;
+          ospf6_prefix_in6_addr (&aselsa->ospf6_prefix, &prefix.prefix);
+          prefix.prefixlen = aselsa->ospf6_prefix.prefix_length;
 
           /* XXX must be reviewed XXX */
           rn2 = route_node_lookup (ospf6->redistribute_map,
@@ -1208,3 +1184,185 @@ ospf6_route_update_zebra ()
     }
 }
 
+
+DEFUN (show_ipv6_route_ospf6_area_detail,
+       show_ipv6_route_ospf6_area_detail_cmd,
+       "show ipv6 route ospf6 area A.B.C.D (detail|)",
+       SHOW_STR
+       IP6_STR
+       ROUTE_STR
+       OSPF6_STR
+       "show route table in area structure\n"
+       "OSPF6 area ID\n"
+       "detailed infomation\n"
+       )
+{
+  struct area *area;
+  area_id_t area_id;
+  struct route_node *rn;
+
+  if (!ospf6)
+    {
+      vty_out (vty, "OSPF6 not started%s", VTY_NEWLINE);
+      return CMD_WARNING;
+    }
+
+  if (argc && strncmp (argv[0], "d", 1) != 0)
+    inet_pton (AF_INET, argv[0], &area_id);
+  else
+    area_id = 0;
+
+  area = ospf6_area_lookup (area_id);
+  if (!area)
+    {
+       vty_out (vty, "no match by area id: %s%s", argv[0],
+		VTY_NEWLINE);
+       return CMD_WARNING;
+    }
+
+  for (rn = route_top (area->table); rn; rn = route_next (rn))
+    {
+      if (rn->info)
+        {
+          if (strncmp (argv[argc-1], "detail", 7) == 0)
+            ospf6_route_vty_new (vty, rn, 1);
+          else
+            ospf6_route_vty_new (vty, rn, 0);
+        }
+    }
+
+  return CMD_SUCCESS;
+}
+
+ALIAS (show_ipv6_route_ospf6_area_detail,
+       show_ipv6_route_ospf6_area_cmd,
+       "show ipv6 route ospf6 area A.B.C.D",
+       SHOW_STR
+       IP6_STR
+       ROUTE_STR
+       OSPF6_STR
+       "show route table in area structure\n"
+       "OSPF6 area ID\n"
+       )
+
+ALIAS (show_ipv6_route_ospf6_area_detail,
+       show_ipv6_route_ospf6_backbone_detail_cmd,
+       "show ipv6 route ospf6 backbone (detail|)",
+       SHOW_STR
+       IP6_STR
+       ROUTE_STR
+       OSPF6_STR
+       "show route table in area structure\n"
+       "detailed infomation\n"
+       )
+
+ALIAS (show_ipv6_route_ospf6_area_detail,
+       show_ipv6_route_ospf6_backbone_cmd,
+       "show ipv6 route ospf6 backbone",
+       SHOW_STR
+       IP6_STR
+       ROUTE_STR
+       OSPF6_STR
+       "show route table in area structure\n"
+       )
+
+DEFUN (show_ipv6_route_ospf6_detail,
+       show_ipv6_route_ospf6_detail_cmd,
+       "show ipv6 route ospf6 (detail|)",
+       SHOW_STR
+       IP6_STR
+       ROUTE_STR
+       OSPF6_STR
+       "detailed infomation\n"
+       )
+{
+  struct route_node *rn;
+  int i, ret, detail;
+  struct prefix p;
+
+  ret = 0;
+  detail = 0;
+
+  if (!ospf6)
+    {
+      vty_out (vty, "OSPF6 not started%s", VTY_NEWLINE);
+      return CMD_WARNING;
+    }
+
+  for (i = 0; i < argc; i++)
+    {
+      if (strncmp (argv[argc-1], "detail", 7) == 0)
+        {
+          detail = 1;
+          continue;
+        }
+
+      ret = str2prefix_ipv6 (argv[i], (struct prefix_ipv6 *) &p);
+      if (ret != 1)
+        continue;
+    }
+
+  if (ret)
+    {
+      rn = route_node_match (ospf6->table, &p);
+      if (rn && rn->info)
+        {
+          ospf6_route_vty_new (vty, rn, 1);
+          route_unlock_node (rn);
+        }
+      else
+        vty_out (vty, "Route not found.%s", VTY_NEWLINE);
+      return CMD_SUCCESS;
+    }
+
+  for (rn = route_top (ospf6->table); rn; rn = route_next (rn))
+    {
+      if (! rn->info)
+        continue;
+
+      ospf6_route_vty_new (vty, rn, detail);
+    }
+
+  return CMD_SUCCESS;
+}
+
+ALIAS (show_ipv6_route_ospf6_detail,
+       show_ipv6_route_ospf6_cmd,
+       "show ipv6 route ospf6",
+       SHOW_STR
+       IP6_STR
+       ROUTE_STR
+       OSPF6_STR
+       )
+
+ALIAS (show_ipv6_route_ospf6_detail,
+       show_ipv6_route_ospf6_prefix_cmd,
+       "show ipv6 route ospf6 X::X",
+       SHOW_STR
+       IP6_STR
+       ROUTE_STR
+       OSPF6_STR
+       "IPv6 Address search\n"
+       )
+
+void
+ospf6_rtable_init ()
+{
+  install_element (VIEW_NODE, &show_ipv6_route_ospf6_cmd);
+  install_element (VIEW_NODE, &show_ipv6_route_ospf6_prefix_cmd);
+  install_element (VIEW_NODE, &show_ipv6_route_ospf6_detail_cmd);
+  install_element (VIEW_NODE, &show_ipv6_route_ospf6_area_cmd);
+  install_element (VIEW_NODE, &show_ipv6_route_ospf6_area_detail_cmd);
+  install_element (VIEW_NODE, &show_ipv6_route_ospf6_backbone_cmd);
+  install_element (VIEW_NODE, &show_ipv6_route_ospf6_backbone_detail_cmd);
+
+  install_element (ENABLE_NODE, &show_ipv6_route_ospf6_cmd);
+  install_element (ENABLE_NODE, &show_ipv6_route_ospf6_prefix_cmd);
+  install_element (ENABLE_NODE, &show_ipv6_route_ospf6_detail_cmd);
+  install_element (ENABLE_NODE, &show_ipv6_route_ospf6_area_cmd);
+  install_element (ENABLE_NODE, &show_ipv6_route_ospf6_area_detail_cmd);
+  install_element (ENABLE_NODE, &show_ipv6_route_ospf6_backbone_cmd);
+  install_element (ENABLE_NODE, &show_ipv6_route_ospf6_backbone_detail_cmd);
+}
+
+
