@@ -35,11 +35,13 @@ struct new_lsdb
   struct
   {
     unsigned long count;
+    unsigned long count_self;
     struct route_table *db;
   } type[OSPF_MAX_LSA];
   unsigned long total;
 };
 
+#if 0
 struct ospf_lsdb
 {
   u_char       flags;
@@ -51,18 +53,25 @@ struct ospf_lsdb
   u_int	       count_self;
   struct ospf_area * area;	/* Associated area */
 };
+#endif
 
 /* Macros. */
-#define EXTERNAL_LSDB(O) \
-        ((O)->external_lsa->type[OSPF_AS_EXTERNAL_LSA].db)
+#define ROUTER_LSDB(A)       ((A)->lsdb->type[OSPF_ROUTER_LSA].db)
+#define NETWORK_LSDB(A)	     ((A)->lsdb->type[OSPF_NETWORK_LSA].db)
+#define SUMMARY_LSDB(A)      ((A)->lsdb->type[OSPF_SUMMARY_LSA].db)
+#define SUMMARY_ASBR_LSDB(A) ((A)->lsdb->type[OSPF_SUMMARY_LSA_ASBR].db)
+#define EXTERNAL_LSDB(O)     ((O)->lsdb->type[OSPF_AS_EXTERNAL_LSA].db)
+#define AREA_LSDB(A,T)       ((A)->lsdb->type[(T)].db)
 
 /* Prototypes. */
-void foreach_lsa (struct route_table *, void *, int,
-		  int (*callback) (struct ospf_lsa *, void *, int));
+struct ospf_lsa *foreach_lsa (struct route_table *, void *, int,
+	              int (*callback) (struct ospf_lsa *, void *, int));
 
 /* New LSDB related functions. */
 struct new_lsdb *new_lsdb_new ();
 void new_lsdb_init (struct new_lsdb *);
+void new_lsdb_free (struct new_lsdb *);
+void new_lsdb_cleanup (struct new_lsdb *);
 void new_lsdb_add (struct new_lsdb *, struct ospf_lsa *);
 struct ospf_lsa *new_lsdb_insert (struct new_lsdb *, struct ospf_lsa *);
 void new_lsdb_delete (struct new_lsdb *, struct ospf_lsa *);
@@ -70,22 +79,9 @@ void new_lsdb_delete_all (struct new_lsdb *);
 struct ospf_lsa *new_lsdb_lookup (struct new_lsdb *, struct ospf_lsa *);
 struct ospf_lsa *new_lsdb_lookup_by_id (struct new_lsdb *, u_char,
 					struct in_addr, struct in_addr);
-unsigned long new_lsdb_count (struct new_lsdb *);
+unsigned long new_lsdb_count_all (struct new_lsdb *);
+unsigned long new_lsdb_count (struct new_lsdb *, int);
+unsigned long new_lsdb_count_self (struct new_lsdb *, int);
 unsigned long new_lsdb_isempty (struct new_lsdb *);
 
-/* Old LSDB related functions. */
-struct ospf_lsdb* ospf_lsdb_new (u_char);
-void ospf_lsdb_free ();
-struct ospf_lsa *ospf_lsdb_add (struct ospf_lsdb *, struct ospf_lsa *);
-void ospf_lsdb_delete (struct ospf_lsdb *, struct ospf_lsa *);
-struct ospf_lsa *ospf_lsdb_lookup (struct ospf_lsdb *, struct in_addr,
-				   struct in_addr);
-struct ospf_lsa *ospf_lsdb_lookup_by_id (struct ospf_lsdb *, struct in_addr);
-struct ospf_lsa *ospf_lsdb_lookup_by_header (struct ospf_lsdb *,
-					     struct ospf_lsa *);
-struct ospf_lsa *ospf_lsdb_iterator (struct ospf_lsdb *, void *, int,
-				     int (*callback) (struct ospf_lsa *, void *, int));
-void id_to_prefix (struct in_addr, struct prefix *);
-void get_lsa_prefix (struct ospf_lsa *, struct prefix *);
-
 #endif /* _ZEBRA_OSPF_LSDB_H */
