@@ -1,7 +1,7 @@
 /*
  * OSPF version 2  Neighbor State Machine
  * From RFC2328 [OSPF Version 2]
- * Copyright (C) 1999 Toshiaki Takada
+ * Copyright (C) 1999, 2000 Toshiaki Takada
  *
  * This file is part of GNU Zebra.
  *
@@ -37,6 +37,7 @@
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_interface.h"
 #include "ospfd/ospf_ism.h"
+#include "ospfd/ospf_asbr.h"
 #include "ospfd/ospf_lsa.h"
 #include "ospfd/ospf_lsdb.h"
 #include "ospfd/ospf_neighbor.h"
@@ -47,7 +48,7 @@
 #include "ospfd/ospf_flood.h"
 #include "ospfd/ospf_abr.h"
 
-extern unsigned long ospf_debug_nsm;
+extern unsigned long term_debug_ospf_nsm;
 
 
 /* OSPF NSM Timer functions. */
@@ -59,9 +60,9 @@ ospf_inactivity_timer (struct thread *thread)
   nbr = THREAD_ARG (thread);
   nbr->t_inactivity = NULL;
 
-  if (IS_OSPF_DEBUG (nsm, NSM_TIMERS))
-    zlog (NULL, LOG_DEBUG, "NSM[%s]: Timer (Inactivity timer expire)",
-	  inet_ntoa (nbr->router_id));
+  if (IS_DEBUG_OSPF (nsm, NSM_TIMERS))
+    zlog (NULL, LOG_DEBUG, "NSM[%s:%s]: Timer (Inactivity timer expire)",
+	  nbr->oi->ifp->name, inet_ntoa (nbr->router_id));
 
   OSPF_NSM_EVENT_SCHEDULE (nbr, NSM_InactivityTimer);
 
@@ -79,9 +80,9 @@ ospf_db_desc_timer (struct thread *thread)
 
   oi = nbr->oi;
 
-  if (IS_OSPF_DEBUG (nsm, NSM_TIMERS))
-    zlog (NULL, LOG_INFO, "NSM[%s]: Timer (DD Retransmit timer expire)",
-	  inet_ntoa (nbr->src));
+  if (IS_DEBUG_OSPF (nsm, NSM_TIMERS))
+    zlog (NULL, LOG_INFO, "NSM[%s:%s]: Timer (DD Retransmit timer expire)",
+	  nbr->oi->ifp->name, inet_ntoa (nbr->src));
 
   /* Sending DD packet. If Last send DD packet remains, re-send it. */
   if (nbr->last_send)
@@ -118,7 +119,9 @@ nsm_timer_set (struct ospf_neighbor *nbr)
       OSPF_NSM_TIMER_ON (nbr->t_db_desc, ospf_db_desc_timer, nbr->v_db_desc);
       break;
     case NSM_Exchange:
-      OSPF_NSM_TIMER_OFF (nbr->t_db_desc);
+      OSPF_NSM_TIMER_ON (nbr->t_ls_upd, ospf_ls_upd_timer, nbr->v_ls_upd);
+      if (!IS_SET_DD_MS (nbr->dd_flags))      
+	OSPF_NSM_TIMER_OFF (nbr->t_db_desc);
       break;
     case NSM_Loading:
       OSPF_NSM_TIMER_OFF (nbr->t_db_desc);
@@ -137,9 +140,9 @@ nsm_timer_set (struct ospf_neighbor *nbr)
 int
 nsm_ignore (struct ospf_neighbor *nbr)
 {
-  if (IS_OSPF_DEBUG (nsm, NSM_EVENTS))
-    zlog (NULL, LOG_INFO, "NSM[%s]: nsm_ignore called",
-	  inet_ntoa (nbr->router_id));
+  if (IS_DEBUG_OSPF (nsm, NSM_EVENTS))
+    zlog (NULL, LOG_INFO, "NSM[%s:%s]: nsm_ignore called",
+	  nbr->oi->ifp->name, inet_ntoa (nbr->router_id));
 
   return 0;
 }
@@ -204,32 +207,64 @@ nsm_twoway_received (struct ospf_neighbor *nbr)
   return next_state;
 }
 
-/* Add LSA to neighbor's summary list. */
+int
+ospf_db_summary_count (struct ospf_neighbor *nbr)
+{
+  return new_lsdb_count_all (&nbr->db_sum);
+}
+
+int
+ospf_db_summary_isempty (struct ospf_neighbor *nbr)
+{
+  return new_lsdb_isempty (&nbr->db_sum);
+}
+
 int
 ospf_db_summary_add (struct ospf_lsa *lsa, void *v, int i)
 {
-  struct ospf_neighbor *nbr;
-
-  if (v == NULL)
-    return 0;
+  struct ospf_neighbor *nbr = (struct ospf_neighbor *) v;
 
   if (lsa == NULL)
     return 0;
 
-  nbr = (struct ospf_neighbor *) v;
-
-  if (LS_AGE (lsa) == OSPF_LSA_MAX_AGE)
+  if (IS_LSA_MAXAGE (lsa))
     {
-      zlog_info ("Z: ospf_db_summary_add() : MaxAge LSA ID: %s", 
-		 inet_ntoa (lsa->data->id));
+      zlog_info ("LSA[Type%d:%s]: LSA is MaxAge, add retransmit list",
+		 lsa->data->id, inet_ntoa (lsa->data->id));
       ospf_ls_retransmit_add (nbr, lsa);                      
     }
   else 
-    list_add_node (nbr->db_summary, lsa);
+    new_lsdb_add (&nbr->db_sum, lsa);
 
   return 0;
 }
 
+#if 0  /* Probably, this functions is not used. */
+void
+ospf_db_summary_delete_all (struct ospf_neighbor *nbr)
+{
+  new_lsdb_delete_all (&nbr->db_sum);
+}
+#endif
+
+void
+ospf_db_summary_clear (struct ospf_neighbor *nbr)
+{
+  struct new_lsdb *lsdb;
+  int i;
+
+  lsdb = &nbr->db_sum;
+  for (i = OSPF_MIN_LSA; i < OSPF_MAX_LSA; i++)
+    {
+      struct route_table *table = lsdb->type[i].db;
+      struct route_node *rn;
+
+      for (rn = route_top (table); rn; rn = route_next (rn))
+	if (rn->info)
+	  new_lsdb_delete (&nbr->db_sum, rn->info);
+    }
+}
+
 
 void nsm_reset_nbr (struct ospf_neighbor *);
 
@@ -246,16 +281,16 @@ nsm_negotiation_done (struct ospf_neighbor *nbr)
 
   area = nbr->oi->area;
 
-  ospf_lsdb_iterator (ROUTER_LSA (area), nbr, 0, ospf_db_summary_add);
-  ospf_lsdb_iterator (NETWORK_LSA (area), nbr, 0, ospf_db_summary_add);
-  ospf_lsdb_iterator (SUMMARY_LSA (area), nbr, 0, ospf_db_summary_add);
-  ospf_lsdb_iterator (SUMMARY_LSA_ASBR (area), nbr, 0, ospf_db_summary_add);
+  foreach_lsa (ROUTER_LSDB (area), nbr, 0, ospf_db_summary_add);
+  foreach_lsa (NETWORK_LSDB (area), nbr, 0, ospf_db_summary_add);
+  foreach_lsa (SUMMARY_LSDB (area), nbr, 0, ospf_db_summary_add);
+  foreach_lsa (SUMMARY_ASBR_LSDB (area), nbr, 0, ospf_db_summary_add);
   
   if (nbr->oi->type != OSPF_IFTYPE_VIRTUALLINK &&
       area->external_routing == OSPF_AREA_DEFAULT)
     foreach_lsa (EXTERNAL_LSDB (ospf_top), nbr, 0, ospf_db_summary_add);
 
-  OSPF_NSM_TIMER_OFF (nbr->t_db_desc);
+  /* OSPF_NSM_TIMER_OFF (nbr->t_db_desc); */
 
   return 0;
 }
@@ -271,7 +306,7 @@ nsm_exchange_done (struct ospf_neighbor *nbr)
     return NSM_Full;
 
   /* Cancel dd retransmit timer. */
-  OSPF_NSM_TIMER_OFF (nbr->t_db_desc);
+  /* OSPF_NSM_TIMER_OFF (nbr->t_db_desc); */
 
   /* Send Link State Request. */
   ospf_ls_req_send (nbr);
@@ -316,8 +351,12 @@ nsm_adj_ok (struct ospf_neighbor *nbr)
     flag = 1;
 
   /* Neighboring Router is the DRouter or the BDRouter. */
+  /*
   if (IPV4_ADDR_SAME (&nbr->address.u.prefix4, &nbr->d_router) ||
       IPV4_ADDR_SAME (&nbr->address.u.prefix4, &nbr->bd_router))
+  */
+  if (IPV4_ADDR_SAME (&nbr->address.u.prefix4, &DR (oi)) ||
+      IPV4_ADDR_SAME (&nbr->address.u.prefix4, &BDR (oi)))
     flag = 1;
 
   if (nbr->status == NSM_TwoWay && flag == 1)
@@ -377,23 +416,19 @@ nsm_oneway_received (struct ospf_neighbor *nbr)
 void
 nsm_reset_nbr (struct ospf_neighbor *nbr)
 {
-  /* Clear Link State Retransmission list. */
-  if (!list_isempty (nbr->ls_retransmit))
-    {
-      ospf_ls_retransmit_clear (nbr);
-      list_delete_all_node (nbr->ls_retransmit);
-    }
-
   /* Clear Database Summary list. */
-  if (!list_isempty (nbr->db_summary))
-    list_delete_all_node (nbr->db_summary);
+  if (!ospf_db_summary_isempty (nbr))
+    ospf_db_summary_clear (nbr);
 
   /* Clear Link State Request list. */
-  if (! ospf_ls_request_isempty (nbr))
+  if (!ospf_ls_request_isempty (nbr))
     ospf_ls_request_delete_all (nbr);
 
+  /* Clear Link State Retransmission list. */
+  if (!ospf_ls_retransmit_isempty (nbr))
+    ospf_ls_retransmit_clear (nbr);
+
   /* Cancel thread. */
-  OSPF_NSM_TIMER_OFF (nbr->t_inactivity);
   OSPF_NSM_TIMER_OFF (nbr->t_db_desc);
   OSPF_NSM_TIMER_OFF (nbr->t_ls_req);
   OSPF_NSM_TIMER_OFF (nbr->t_ls_upd);
@@ -623,9 +658,9 @@ nsm_change_status (struct ospf_neighbor *nbr, int status)
   struct ospf_interface *oi;
 
   /* Logging change of status. */
-  if (IS_OSPF_DEBUG (nsm, NSM_STATUS))
-    zlog_info ("NSM Status change [%s:%s] %s -> %s",
-	       inet_ntoa (nbr->router_id), nbr->oi->ifp->name,
+  if (IS_DEBUG_OSPF (nsm, NSM_STATUS))
+    zlog_info ("NSM[%s:%s]: Status change %s -> %s",
+	       nbr->oi->ifp->name, inet_ntoa (nbr->router_id),
 	       LOOKUP (ospf_nsm_status_msg, nbr->status),
 	       LOOKUP (ospf_nsm_status_msg, status));
 
@@ -667,25 +702,26 @@ nsm_change_status (struct ospf_neighbor *nbr, int status)
 		ospf_schedule_abr_task ();
 	}
 
-      zlog_info ("Z:nsm_change_status(): "
+      zlog_info ("Z: nsm_change_status(): "
 		 "scheduling new router-LSA origination");
 
-      ospf_schedule_router_lsa_originate (oi->area);
+      ospf_router_lsa_timer_add (oi->area);
 
       if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
-	ospf_schedule_router_lsa_originate (oi->vl_data->vl_area);
+	ospf_router_lsa_timer_add (oi->vl_data->vl_area);
 
       /* Originate network-LSA. */
       if (oi->status == ISM_DR)
 	{
-/*	  thread_execute (master, ospf_network_lsa_refresh, oi, 0); */
 	  if (oi->network_lsa_self && oi->full_nbrs == 0)
 	    {
 	      ospf_lsa_flush_area (oi->network_lsa_self, oi->area);
+	      ospf_lsa_unlock (oi->network_lsa_self);
 	      oi->network_lsa_self = NULL;
+	      OSPF_TIMER_OFF (oi->t_network_lsa_self);
 	    }
 	  else
-	    ospf_schedule_network_lsa_originate (oi);
+	    ospf_network_lsa_timer_add (oi);
 	}
     }
     
@@ -722,8 +758,9 @@ ospf_nsm_event (struct thread *thread)
      deleted. */
   if (event == NSM_KillNbr || event == NSM_InactivityTimer)
     {
-      if (IS_OSPF_DEBUG (nsm, NSM_EVENTS))
-	zlog_info ("NSM[%s]: the neighbor is deleted", inet_ntoa (router_id));
+      if (IS_DEBUG_OSPF (nsm, NSM_EVENTS))
+	zlog_info ("NSM[%s:%s]: neighbor deleted",
+		   oi->ifp->name, inet_ntoa (router_id));
 
       /* Timers are canceled in ospf_nbr_free, moreover we cannot call
          nsm_timer_set here because nbr is freed already!!!*/
@@ -735,8 +772,9 @@ ospf_nsm_event (struct thread *thread)
   if (! next_state)
     next_state = NSM [nbr->status][event].next_state;
 
-  if (IS_OSPF_DEBUG (nsm, NSM_EVENTS))
-    zlog_info ("NSM[%s]: %s (%s)", inet_ntoa (nbr->router_id),
+  if (IS_DEBUG_OSPF (nsm, NSM_EVENTS))
+    zlog_info ("NSM[%s:%s]: %s (%s)", oi->ifp->name,
+	       inet_ntoa (nbr->router_id),
 	       LOOKUP (ospf_nsm_status_msg, nbr->status),
 	       ospf_nsm_event_str [event]);
   
