@@ -25,13 +25,17 @@
 
 #include "table.h"
 
-void zebra_redistribute_add (int, struct zebra_client *, int);
-void zebra_redistribute_delete (int, struct zebra_client *, int);
+void zebra_redistribute_add (int, struct zserv *, int);
+void zebra_redistribute_delete (int, struct zserv *, int);
+
+void zebra_redistribute_default_add (int, struct zserv *, int);
+void zebra_redistribute_default_delete (int, struct zserv *, int);
 
 void redistribute_add (struct route_node *np, struct rib *rib);
 void redistribute_delete (struct route_node *np, struct rib *rib);
 
 void zebra_interface_add_update (struct interface *ifp);
+void zebra_interface_delete_update (struct interface *ifp);
 void zebra_interface_address_add_update (struct interface *ifp, 
 					 struct connected *c);
 
